@@ -20,7 +20,7 @@
 import os
 import os.path
 import tempfile
-import logging
+from ipapython.ipa_log_manager import *
 import pwd
 import shutil
 
@@ -31,16 +31,18 @@ import installutils
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython import services as ipaservices
+from ipapython import dogtag
 from ipalib import util, api
 
 HTTPD_DIR = "/etc/httpd"
 SSL_CONF = HTTPD_DIR + "/conf.d/ssl.conf"
 NSS_CONF = HTTPD_DIR + "/conf.d/nss.conf"
 
-selinux_warning = """WARNING: could not set selinux boolean httpd_can_network_connect to true.
-The web interface may not function correctly until this boolean is
-successfully change with the command:
-   /usr/sbin/setsebool -P httpd_can_network_connect true
+selinux_warning = """
+WARNING: could not set selinux boolean(s) %(var)s to true.  The web
+interface may not function correctly until this boolean is successfully
+change with the command:
+   /usr/sbin/setsebool -P %(var)s true
 Try updating the policycoreutils and selinux-policy packages.
 """
 
@@ -50,24 +52,32 @@ class WebGuiInstance(service.SimpleServiceInstance):
 
 class HTTPInstance(service.Service):
     def __init__(self, fstore = None):
-        service.Service.__init__(self, "httpd")
+        service.Service.__init__(self, "httpd", service_desc="the web interface")
         if fstore:
             self.fstore = fstore
         else:
             self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
 
+    subject_base = ipautil.dn_attribute_property('_subject_base')
+
     def create_instance(self, realm, fqdn, domain_name, dm_password=None, autoconfig=True, pkcs12_info=None, self_signed_ca=False, subject_base=None, auto_redirect=True):
         self.fqdn = fqdn
         self.realm = realm
         self.domain = domain_name
         self.dm_password = dm_password
-        self.suffix = util.realm_to_suffix(self.realm)
+        self.suffix = ipautil.realm_to_suffix(self.realm)
         self.pkcs12_info = pkcs12_info
         self.self_signed_ca = self_signed_ca
         self.principal = "HTTP/%s@%s" % (self.fqdn, self.realm)
         self.dercert = None
         self.subject_base = subject_base
-        self.sub_dict = {"REALM": realm, "FQDN": fqdn, "DOMAIN": self.domain, "AUTOREDIR": '' if auto_redirect else '#'}
+        self.sub_dict = dict(
+            REALM=realm,
+            FQDN=fqdn,
+            DOMAIN=self.domain,
+            AUTOREDIR='' if auto_redirect else '#',
+            CRL_PUBLISH_PATH=dogtag.install_constants.CRL_PUBLISH_PATH,
+        )
 
         # get a connection to the DS
         self.ldap_connect()
@@ -84,11 +94,12 @@ class HTTPInstance(service.Service):
             self.step("setting up browser autoconfig", self.__setup_autoconfig)
         self.step("publish CA cert", self.__publish_ca_cert)
         self.step("creating a keytab for httpd", self.__create_http_keytab)
-        self.step("configuring SELinux for httpd", self.__selinux_config)
+        self.step("clean up any existing httpd ccache", self.remove_httpd_ccache)
+        self.step("configuring SELinux for httpd", self.configure_selinux_for_httpd)
         self.step("restarting httpd", self.__start)
         self.step("configuring httpd to start on boot", self.__enable)
 
-        self.start_creation("Configuring the web interface", 60)
+        self.start_creation(runtime=60)
 
     def __start(self):
         self.backup_state("running", self.is_running())
@@ -101,31 +112,68 @@ class HTTPInstance(service.Service):
         # components as found in our LDAP configuration tree
         self.ldap_enable('HTTP', self.fqdn, self.dm_password, self.suffix)
 
-    def __selinux_config(self):
-        selinux=0
+    def configure_selinux_for_httpd(self):
+        def get_setsebool_args(changes):
+            if len(changes) == 1:
+                # workaround https://bugzilla.redhat.com/show_bug.cgi?id=825163
+                updates = changes.items()[0]
+            else:
+                updates = ["%s=%s" % update for update in changes.iteritems()]
+
+            args = ["/usr/sbin/setsebool", "-P"]
+            args.extend(updates)
+
+            return args
+
+        selinux = False
         try:
             if (os.path.exists('/usr/sbin/selinuxenabled')):
                 ipautil.run(["/usr/sbin/selinuxenabled"])
-                selinux=1
+                selinux = True
         except ipautil.CalledProcessError:
             # selinuxenabled returns 1 if not enabled
             pass
 
         if selinux:
-            try:
-                # returns e.g. "httpd_can_network_connect --> off"
-                (stdout, stderr, returncode) = ipautil.run(["/usr/sbin/getsebool",
-                                                "httpd_can_network_connect"])
-                self.backup_state("httpd_can_network_connect", stdout.split()[2])
-            except:
-                pass
+            # Don't assume all vars are available
+            updated_vars = {}
+            failed_vars = {}
+            required_settings = (("httpd_can_network_connect", "on"),
+                                 ("httpd_manage_ipa", "on"))
+            for setting, state in required_settings:
+                try:
+                    (stdout, stderr, returncode) = ipautil.run(["/usr/sbin/getsebool", setting])
+                    original_state = stdout.split()[2]
+                    self.backup_state(setting, original_state)
 
-            # Allow apache to connect to the turbogears web gui
-            # This can still fail even if selinux is enabled
-            try:
-                ipautil.run(["/usr/sbin/setsebool", "-P", "httpd_can_network_connect", "true"])
-            except:
-                self.print_msg(selinux_warning)
+                    if original_state != state:
+                        updated_vars[setting] = state
+                except ipautil.CalledProcessError, e:
+                    root_logger.debug("Cannot get SELinux boolean '%s': %s", setting, e)
+                    failed_vars[setting] = state
+
+            # Allow apache to connect to the dogtag UI and the session cache
+            # This can still fail even if selinux is enabled. Execute these
+            # together so it is speedier.
+            if updated_vars:
+                args = get_setsebool_args(updated_vars)
+                try:
+                    ipautil.run(args)
+                except ipautil.CalledProcessError:
+                    failed_vars.update(updated_vars)
+
+            if failed_vars:
+                args = get_setsebool_args(failed_vars)
+                names = [update[0] for update in updated_vars]
+                message = ['WARNING: could not set the following SELinux boolean(s):']
+                for update in failed_vars.iteritems():
+                    message.append('  %s -> %s' % update)
+                message.append('The web interface may not function correctly until the booleans')
+                message.append('are successfully changed with the command:')
+                message.append(' '.join(args))
+                message.append('Try updating the policycoreutils and selinux-policy packages.')
+
+                self.print_msg("\n".join(message))
 
     def __create_http_keytab(self):
         installutils.kadmin_addprinc(self.principal)
@@ -136,6 +184,11 @@ class HTTPInstance(service.Service):
         pent = pwd.getpwnam("apache")
         os.chown("/etc/httpd/conf/ipa.keytab", pent.pw_uid, pent.pw_gid)
 
+    def remove_httpd_ccache(self):
+        # Clean up existing ccache
+        pent = pwd.getpwnam("apache")
+        installutils.remove_file('/tmp/krb5cc_%d' % pent.pw_uid)
+
     def __configure_http(self):
         target_fname = '/etc/httpd/conf.d/ipa.conf'
         http_txt = ipautil.template_file(ipautil.SHARE_DIR + "ipa.conf", self.sub_dict)
@@ -196,7 +249,7 @@ class HTTPInstance(service.Service):
             # We only handle one server cert
             nickname = server_certs[0][0]
             self.dercert = db.get_cert_from_db(nickname, pem=False)
-            db.track_server_cert(nickname, self.principal, db.passwd_fname)
+            db.track_server_cert(nickname, self.principal, db.passwd_fname, 'restart_httpd')
 
             self.__set_mod_nss_nickname(nickname)
         else:
@@ -205,7 +258,7 @@ class HTTPInstance(service.Service):
 
             db.create_password_conf()
             self.dercert = db.create_server_cert("Server-Cert", self.fqdn, ca_db)
-            db.track_server_cert("Server-Cert", self.principal, db.passwd_fname)
+            db.track_server_cert("Server-Cert", self.principal, db.passwd_fname, 'restart_httpd')
             db.create_signing_cert("Signing-Cert", "Object Signing Cert", ca_db)
 
         # Fix the database permissions
@@ -233,20 +286,18 @@ class HTTPInstance(service.Service):
 
     def __setup_autoconfig(self):
         target_fname = '/usr/share/ipa/html/preferences.html'
-        prefs_txt = ipautil.template_file(ipautil.SHARE_DIR + "preferences.html.template", self.sub_dict)
-        prefs_fd = open(target_fname, "w")
-        prefs_fd.write(prefs_txt)
-        prefs_fd.close()
+        ipautil.copy_template_file(
+            ipautil.SHARE_DIR + "preferences.html.template",
+            target_fname, self.sub_dict)
         os.chmod(target_fname, 0644)
 
         # The signing cert is generated in __setup_ssl
         db = certs.CertDB(self.realm, subject_base=self.subject_base)
+        with open(db.passwd_fname) as pwdfile:
+            pwd = pwdfile.read()
 
-        pwdfile = open(db.passwd_fname)
-        pwd = pwdfile.read()
-        pwdfile.close()
-
-        tmpdir = tempfile.mkdtemp(prefix = "tmp-")
+        # Setup configure.jar
+        tmpdir = tempfile.mkdtemp(prefix="tmp-")
         target_fname = '/usr/share/ipa/html/configure.jar'
         shutil.copy("/usr/share/ipa/html/preferences.html", tmpdir)
         db.run_signtool(["-k", "Signing-Cert",
@@ -256,6 +307,50 @@ class HTTPInstance(service.Service):
         shutil.rmtree(tmpdir)
         os.chmod(target_fname, 0644)
 
+        self.setup_firefox_extension(self.realm, self.domain, force=True)
+
+    def setup_firefox_extension(self, realm, domain, force=False):
+        """Set up the signed browser configuration extension
+
+        If the extension is already set up, skip the installation unless
+        ``force`` is true.
+        """
+
+        target_fname = '/usr/share/ipa/html/krb.js'
+        if os.path.exists(target_fname) and not force:
+            root_logger.info(
+                '%s exists, skipping install of Firefox extension',
+                    target_fname)
+            return
+
+        sub_dict = dict(REALM=realm, DOMAIN=domain)
+        db = certs.CertDB(realm)
+        with open(db.passwd_fname) as pwdfile:
+            pwd = pwdfile.read()
+
+        ipautil.copy_template_file(ipautil.SHARE_DIR + "krb.js.template",
+            target_fname, sub_dict)
+        os.chmod(target_fname, 0644)
+
+        # Setup extension
+        tmpdir = tempfile.mkdtemp(prefix="tmp-")
+        extdir = tmpdir + "/ext"
+        target_fname = "/usr/share/ipa/html/kerberosauth.xpi"
+        shutil.copytree("/usr/share/ipa/ffextension", extdir)
+        if db.has_nickname('Signing-Cert'):
+            db.run_signtool(["-k", "Signing-Cert",
+                                "-p", pwd,
+                                "-X", "-Z", target_fname,
+                                extdir])
+        else:
+            root_logger.warning('Object-signing certificate was not found. '
+                'Creating unsigned Firefox configuration extension.')
+            filenames = os.listdir(extdir)
+            ipautil.run(['/usr/bin/zip', '-r', target_fname] + filenames,
+                cwd=extdir)
+        shutil.rmtree(tmpdir)
+        os.chmod(target_fname, 0644)
+
     def __publish_ca_cert(self):
         ca_db = certs.CertDB(self.realm)
         ca_db.publish_ca_cert("/usr/share/ipa/html/ca.crt")
@@ -279,7 +374,7 @@ class HTTPInstance(service.Service):
             try:
                 self.fstore.restore_file(f)
             except ValueError, error:
-                logging.debug(error)
+                root_logger.debug(error)
                 pass
 
         # Remove the configuration files we create
@@ -287,12 +382,14 @@ class HTTPInstance(service.Service):
         installutils.remove_file("/etc/httpd/conf.d/ipa.conf")
         installutils.remove_file("/etc/httpd/conf.d/ipa-pki-proxy.conf")
 
-        sebool_state = self.restore_state("httpd_can_network_connect")
-        if not sebool_state is None:
-            try:
-                ipautil.run(["/usr/sbin/setsebool", "-P", "httpd_can_network_connect", sebool_state])
-            except:
-                self.print_msg(selinux_warning)
+        for var in ["httpd_can_network_connect", "httpd_manage_ipa"]:
+            sebool_state = self.restore_state(var)
+            if not sebool_state is None:
+                try:
+                    ipautil.run(["/usr/sbin/setsebool", "-P", var, sebool_state])
+                except ipautil.CalledProcessError, e:
+                    self.print_msg("Cannot restore SELinux boolean '%s' back to '%s': %s" \
+                            % (var, sebool_state, e))
 
         if not running is None and running:
             self.start()
