@@ -18,7 +18,6 @@
 #
 
 import shutil
-import logging
 import fileinput
 import re
 import sys
@@ -33,6 +32,8 @@ from ipapython import ipautil
 from ipapython import services as ipaservices
 from ipalib import util
 from ipalib import errors
+from ipapython.ipa_log_manager import *
+from ipapython.dn import DN
 
 from ipaserver import ipaldap
 from ipaserver.install import replication
@@ -50,8 +51,6 @@ import struct
 import certs
 from distutils import version
 
-KRBMKEY_DENY_ACI = '(targetattr = "krbMKey")(version 3.0; acl "No external access"; deny (read,write,search,compare) userdn != "ldap:///uid=kdc,cn=sysaccounts,cn=etc,$SUFFIX";)'
-
 def update_key_val_in_file(filename, key, val):
     if os.path.exists(filename):
         pattern = "^[\s#]*%s\s*=\s*%s\s*" % (re.escape(key), re.escape(val))
@@ -74,11 +73,11 @@ def update_key_val_in_file(filename, key, val):
 
 class KpasswdInstance(service.SimpleServiceInstance):
     def __init__(self):
-        service.SimpleServiceInstance.__init__(self, "ipa_kpasswd")
+        service.SimpleServiceInstance.__init__(self, "kadmin")
 
 class KrbInstance(service.Service):
     def __init__(self, fstore=None):
-        service.Service.__init__(self, "krb5kdc")
+        service.Service.__init__(self, "krb5kdc", service_desc="Kerberos KDC")
         self.fqdn = None
         self.realm = None
         self.domain = None
@@ -86,6 +85,7 @@ class KrbInstance(service.Service):
         self.admin_password = None
         self.master_password = None
         self.suffix = None
+        self.subject_base = None
         self.kdc_password = None
         self.sub_dict = None
         self.pkcs12_info = None
@@ -96,23 +96,30 @@ class KrbInstance(service.Service):
         else:
             self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
 
+    suffix = ipautil.dn_attribute_property('_suffix')
+    subject_base = ipautil.dn_attribute_property('_subject_base')
+
+    def get_realm_suffix(self):
+        return DN(('cn', self.realm), ('cn', 'kerberos'), self.suffix)
+
     def move_service_to_host(self, principal):
         """
         Used to move a host/ service principal created by kadmin.local from
         cn=kerberos to reside under the host entry.
         """
 
-        service_dn = "krbprincipalname=%s,cn=%s,cn=kerberos,%s" % (principal, self.realm, self.suffix)
+        service_dn = DN(('krbprincipalname', principal), self.get_realm_suffix())
         service_entry = self.admin_conn.getEntry(service_dn, ldap.SCOPE_BASE)
         self.admin_conn.deleteEntry(service_dn)
 
         # Create a host entry for this master
-        host_dn = "fqdn=%s,cn=computers,cn=accounts,%s" % (self.fqdn, self.suffix)
+        host_dn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
         host_entry = ipaldap.Entry(host_dn)
-        host_entry.setValues('objectclass', ['top', 'ipaobject', 'nshost', 'ipahost', 'ipaservice', 'pkiuser', 'krbprincipalaux', 'krbprincipal', 'krbticketpolicyaux'])
+        host_entry.setValues('objectclass', ['top', 'ipaobject', 'nshost', 'ipahost', 'ipaservice', 'pkiuser', 'krbprincipalaux', 'krbprincipal', 'krbticketpolicyaux', 'ipasshhost'])
         host_entry.setValues('krbextradata', service_entry.getValues('krbextradata'))
         host_entry.setValue('krblastpwdchange', service_entry.getValue('krblastpwdchange'))
-        host_entry.setValue('krbpasswordexpiration', service_entry.getValue('krbpasswordexpiration'))
+        if 'krbpasswordexpiration' in service_entry.toDict():
+            host_entry.setValue('krbpasswordexpiration', service_entry.getValue('krbpasswordexpiration'))
         host_entry.setValue('krbprincipalname', service_entry.getValue('krbprincipalname'))
         if 'krbticketflags' in service_entry.toDict():
             host_entry.setValue('krbticketflags', service_entry.getValue('krbticketflags'))
@@ -130,7 +137,7 @@ class KrbInstance(service.Service):
         self.host = host_name.split(".")[0]
         self.ip = socket.getaddrinfo(host_name, None, socket.AF_UNSPEC, socket.SOCK_STREAM)[0][4][0]
         self.domain = domain_name
-        self.suffix = util.realm_to_suffix(self.realm)
+        self.suffix = ipautil.realm_to_suffix(self.realm)
         self.kdc_password = ipautil.ipa_generate_password()
         self.admin_password = admin_password
         self.dm_password = admin_password
@@ -159,47 +166,40 @@ class KrbInstance(service.Service):
 
         self.__common_setup(realm_name, host_name, domain_name, admin_password)
 
-        self.step("setting KDC account password", self.__configure_kdc_account_password)
         self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
-        self.step("adding kerberos entries to the DS", self.__add_krb_entries)
+        self.step("adding kerberos container to the directory", self.__add_krb_container)
+        self.step("configuring KDC", self.__configure_instance)
+        self.step("initialize kerberos container", self.__init_ipa_kdb)
         self.step("adding default ACIs", self.__add_default_acis)
-        self.step("configuring KDC", self.__create_instance)
-        self.step("adding default keytypes", self.__add_default_keytypes)
-        self.step("adding default password policy", self.__add_default_pwpolicy)
         self.step("creating a keytab for the directory", self.__create_ds_keytab)
         self.step("creating a keytab for the machine", self.__create_host_keytab)
-        self.step("exporting the kadmin keytab", self.__export_kadmin_changepw_keytab)
         self.step("adding the password extension to the directory", self.__add_pwd_extop_module)
-        self.step("adding the kerberos master key to the directory", self.__add_master_key)
         if setup_pkinit:
             self.step("creating X509 Certificate for PKINIT", self.__setup_pkinit)
             self.step("creating principal for anonymous PKINIT", self.__add_anonymous_pkinit_principal)
 
         self.__common_post_setup()
 
-        self.start_creation("Configuring Kerberos KDC", 30)
+        self.start_creation(runtime=30)
 
         self.kpasswd = KpasswdInstance()
-        self.kpasswd.create_instance('KPASSWD', self.fqdn, self.admin_password, self.suffix)
+        self.kpasswd.create_instance('KPASSWD', self.fqdn, self.admin_password, self.suffix, realm=self.realm)
 
     def create_replica(self, realm_name,
                        master_fqdn, host_name,
                        domain_name, admin_password,
-                       ldap_passwd_filename, kpasswd_filename,
                        setup_pkinit=False, pkcs12_info=None,
                        self_signed_ca=False, subject_base=None):
         self.pkcs12_info = pkcs12_info
         self.self_signed_ca = self_signed_ca
         self.subject_base = subject_base
-        self.__copy_ldap_passwd(ldap_passwd_filename)
-        self.__copy_kpasswd_keytab(kpasswd_filename)
         self.master_fqdn = master_fqdn
 
         self.__common_setup(realm_name, host_name, domain_name, admin_password)
 
         self.step("adding sasl mappings to the directory", self.__configure_sasl_mappings)
         self.step("writing stash file from DS", self.__write_stash_from_ds)
-        self.step("configuring KDC", self.__create_replica_instance)
+        self.step("configuring KDC", self.__configure_instance)
         self.step("creating a keytab for the directory", self.__create_ds_keytab)
         self.step("creating a keytab for the machine", self.__create_host_keytab)
         self.step("adding the password extension to the directory", self.__add_pwd_extop_module)
@@ -209,31 +209,11 @@ class KrbInstance(service.Service):
 
         self.__common_post_setup()
 
-        self.start_creation("Configuring Kerberos KDC", 30)
+        self.start_creation(runtime=30)
 
         self.kpasswd = KpasswdInstance()
         self.kpasswd.create_instance('KPASSWD', self.fqdn, self.admin_password, self.suffix)
 
-    def __copy_ldap_passwd(self, filename):
-        self.fstore.backup_file("/var/kerberos/krb5kdc/ldappwd")
-        shutil.copy(filename, "/var/kerberos/krb5kdc/ldappwd")
-        os.chmod("/var/kerberos/krb5kdc/ldappwd", 0600)
-
-    def __copy_kpasswd_keytab(self, filename):
-        self.fstore.backup_file("/var/kerberos/krb5kdc/kpasswd.keytab")
-        shutil.copy(filename, "/var/kerberos/krb5kdc/kpasswd.keytab")
-        os.chmod("/var/kerberos/krb5kdc/kpasswd.keytab", 0600)
-
-
-    def __configure_kdc_account_password(self):
-        hexpwd = ''
-        for x in self.kdc_password:
-            hexpwd += (hex(ord(x))[2:])
-        self.fstore.backup_file("/var/kerberos/krb5kdc/ldappwd")
-        pwd_fd = open("/var/kerberos/krb5kdc/ldappwd", "w")
-        pwd_fd.write("uid=kdc,cn=sysaccounts,cn=etc,"+self.suffix+"#{HEX}"+hexpwd+"\n")
-        pwd_fd.close()
-        os.chmod("/var/kerberos/krb5kdc/ldappwd", 0600)
 
     def __enable(self):
         self.backup_state("enabled", self.is_enabled())
@@ -246,7 +226,7 @@ class KrbInstance(service.Service):
         try:
             self.start()
         except:
-            logging.critical("krb5kdc service failed to start")
+            root_logger.critical("krb5kdc service failed to start")
 
     def __setup_sub_dict(self):
         self.sub_dict = dict(FQDN=self.fqdn,
@@ -258,25 +238,38 @@ class KrbInstance(service.Service):
                              SERVER_ID=dsinstance.realm_to_serverid(self.realm),
                              REALM=self.realm)
 
+        # IPA server/KDC is not a subdomain of default domain
+        # Proper domain-realm mapping needs to be specified
+        dr_map = ''
+        if not self.fqdn.endswith(self.domain):
+            root_logger.debug("IPA FQDN '%s' is not located in default domain '%s'" \
+                    % (self.fqdn, self.domain))
+            server_host, dot, server_domain = self.fqdn.partition('.')
+            root_logger.debug("Domain '%s' needs additional mapping in krb5.conf" \
+                    % server_domain)
+            dr_map = " .%(domain)s = %(realm)s\n %(domain)s = %(realm)s\n" \
+                        % dict(domain=server_domain, realm=self.realm)
+        self.sub_dict['OTHER_DOMAIN_REALM_MAPS'] = dr_map
+
     def __configure_sasl_mappings(self):
         # we need to remove any existing SASL mappings in the directory as otherwise they
         # they may conflict.
 
         try:
-            res = self.admin_conn.search_s("cn=mapping,cn=sasl,cn=config",
+            res = self.admin_conn.search_s(DN(('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')),
                                      ldap.SCOPE_ONELEVEL,
                                      "(objectclass=nsSaslMapping)")
             for r in res:
                 try:
                     self.admin_conn.delete_s(r.dn)
                 except LDAPError, e:
-                    logging.critical("Error during SASL mapping removal: %s" % str(e))
+                    root_logger.critical("Error during SASL mapping removal: %s" % str(e))
                     raise e
         except LDAPError, e:
-            logging.critical("Error while enumerating SASL mappings %s" % str(e))
+            root_logger.critical("Error while enumerating SASL mappings %s" % str(e))
             raise e
 
-        entry = ipaldap.Entry("cn=Full Principal,cn=mapping,cn=sasl,cn=config")
+        entry = ipaldap.Entry(DN(('cn', 'Full Principal'), ('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')))
         entry.setValues("objectclass", "top", "nsSaslMapping")
         entry.setValues("cn", "Full Principal")
         entry.setValues("nsSaslMapRegexString", '\(.*\)@\(.*\)')
@@ -284,12 +277,12 @@ class KrbInstance(service.Service):
         entry.setValues("nsSaslMapFilterTemplate", '(krbPrincipalName=\\1@\\2)')
 
         try:
-            self.admin_conn.add_s(entry)
+            self.admin_conn.addEntry(entry)
         except ldap.ALREADY_EXISTS:
-            logging.critical("failed to add Full Principal Sasl mapping")
+            root_logger.critical("failed to add Full Principal Sasl mapping")
             raise e
 
-        entry = ipaldap.Entry("cn=Name Only,cn=mapping,cn=sasl,cn=config")
+        entry = ipaldap.Entry(DN(('cn', 'Name Only'), ('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')))
         entry.setValues("objectclass", "top", "nsSaslMapping")
         entry.setValues("cn", "Name Only")
         entry.setValues("nsSaslMapRegexString", '^[^:@]+$')
@@ -297,26 +290,17 @@ class KrbInstance(service.Service):
         entry.setValues("nsSaslMapFilterTemplate", '(krbPrincipalName=&@%s)' % self.realm)
 
         try:
-            self.admin_conn.add_s(entry)
+            self.admin_conn.addEntry(entry)
         except ldap.ALREADY_EXISTS:
-            logging.critical("failed to add Name Only Sasl mapping")
+            root_logger.critical("failed to add Name Only Sasl mapping")
             raise e
 
-    def __add_krb_entries(self):
+    def __add_krb_container(self):
         self._ldap_mod("kerberos.ldif", self.sub_dict)
 
     def __add_default_acis(self):
         self._ldap_mod("default-aci.ldif", self.sub_dict)
 
-    def __add_default_keytypes(self):
-        self._ldap_mod("default-keytypes.ldif", self.sub_dict)
-
-    def __add_default_pwpolicy(self):
-        self._ldap_mod("default-pwpolicy.ldif", self.sub_dict)
-
-    def __create_replica_instance(self):
-        self.__create_instance(replica=True)
-
     def __template_file(self, path, chmod=0644):
         template = os.path.join(ipautil.SHARE_DIR, os.path.basename(path) + ".template")
         conf = ipautil.template_file(template, self.sub_dict)
@@ -327,29 +311,29 @@ class KrbInstance(service.Service):
         if chmod is not None:
             os.chmod(path, chmod)
 
-    def __create_instance(self, replica=False):
+    def __init_ipa_kdb(self):
+        #populate the directory with the realm structure
+        args = ["kdb5_util", "create", "-s",
+                                       "-r", self.realm,
+                                       "-x", "ipa-setup-override-restrictions"]
+        dialogue = (
+            # Enter KDC database master key:
+            self.master_password + '\n',
+            # Re-enter KDC database master key to verify:
+            self.master_password + '\n',
+        )
+        try:
+            ipautil.run(args, nolog=(self.master_password,), stdin=''.join(dialogue))
+        except ipautil.CalledProcessError, e:
+            print "Failed to initialize the realm container"
+
+    def __configure_instance(self):
         self.__template_file("/var/kerberos/krb5kdc/kdc.conf", chmod=None)
         self.__template_file("/etc/krb5.conf")
         self.__template_file("/usr/share/ipa/html/krb5.ini")
         self.__template_file("/usr/share/ipa/html/krb.con")
         self.__template_file("/usr/share/ipa/html/krbrealm.con")
 
-        if not replica:
-            #populate the directory with the realm structure
-            args = ["kdb5_ldap_util", "-D", "uid=kdc,cn=sysaccounts,cn=etc,"+self.suffix, "create", "-s", "-r", self.realm, "-subtrees", self.suffix, "-sscope", "sub"]
-            dialogue = (
-                # Password for "uid=kdc,cn=sysaccounts,cn=etc,...":
-                self.kdc_password + '\n',
-                # Enter KDC database master key:
-                self.master_password + '\n',
-                # Re-enter KDC database master key to verify:
-                self.master_password + '\n',
-            )
-            try:
-                ipautil.run(args, nolog=(self.kdc_password, self.master_password), stdin=''.join(dialogue))
-            except ipautil.CalledProcessError, e:
-                print "Failed to populate the realm structure in kerberos", e
-
         MIN_KRB5KDC_WITH_WORKERS = "1.9"
         cpus = os.sysconf('SC_NPROCESSORS_ONLN')
         workers = False
@@ -373,12 +357,13 @@ class KrbInstance(service.Service):
 
     def __write_stash_from_ds(self):
         try:
-            entry = self.admin_conn.getEntry("cn=%s, cn=kerberos, %s" % (self.realm, self.suffix), ldap.SCOPE_SUBTREE)
+            entry = self.admin_conn.getEntry(self.get_realm_suffix(),
+                                             ldap.SCOPE_SUBTREE)
         except errors.NotFound, e:
-            logging.critical("Could not find master key in DS")
+            root_logger.critical("Could not find master key in DS")
             raise e
 
-        krbMKey = pyasn1.codec.ber.decoder.decode(entry.krbmkey)
+        krbMKey = pyasn1.codec.ber.decoder.decode(entry.getValue('krbmkey'))
         keytype = int(krbMKey[0][1][0])
         keydata = str(krbMKey[0][1][1])
 
@@ -389,97 +374,13 @@ class KrbInstance(service.Service):
             fd.write(s)
             fd.close()
         except os.error, e:
-            logging.critical("failed to write stash file")
+            root_logger.critical("failed to write stash file")
             raise e
 
     #add the password extop module
     def __add_pwd_extop_module(self):
         self._ldap_mod("pwd-extop-conf.ldif", self.sub_dict)
 
-    def __add_master_key(self):
-        #check for a keytab file by checking if the header magic is for a keytab
-        def __is_keytab(header):
-            if header == 0x0502 or header == 0x0501 or header == 0x0205 or header == 0x0105:
-                return 1
-            else:
-                return 0
-        #check whether a keytab file is v1 or v2
-        def __keytab_version(header):
-            if header == 0x0502 or header == 0x0205:
-                return 2
-            elif header == 0x0501 or header == 0x0105:
-                return 1
-            else:
-                return 0
-        #get the Master Key from the stash file
-        try:
-            stash = open("/var/kerberos/krb5kdc/.k5."+self.realm, "r")
-            keytype = struct.unpack('h', stash.read(2))[0]
-            if __is_keytab(keytype):
-                #in v2, all numbers are stored in network order
-                if __keytab_version(keytype) > 1:
-                    __endian = '!'
-                else:
-                    __endian = ''
-                #walk the first entry (there should only be one)
-                keyentrylen = struct.unpack(__endian + 'i', stash.read(4))[0]
-                #number of components in the principal name
-                keyprinccomps = struct.unpack(__endian + 'h', stash.read(2))[0]
-                #version 1 counted the realm as a component, version 2 doesn't
-                if __keytab_version(keytype) == 1:
-                    keyprinccomps = keyprinccomps - 1
-                keyprinc = []
-                #read the components. the realm goes first, so we should
-                #end up with (realm, "K", "M")
-                for i in range(keyprinccomps + 1):
-                    keyprinccompsize = struct.unpack(__endian + 'h', stash.read(2))[0]
-                    keyprinc = keyprinc + [stash.read(keyprinccompsize)]
-                #version 2 added the principal name type, otherwise we just
-                #assume it's a regular old principal name
-                if __keytab_version(keytype) > 1:
-                    keyprinctype = struct.unpack(__endian + 'i', stash.read(4))[0]
-                else:
-                    keyprinctype = 1
-                #date the key was added to this keytab
-                keydate = struct.unpack(__endian + 'i', stash.read(4))[0]
-                #kvno
-                keyversion = struct.unpack('B', stash.read(1))[0]
-                #read the real enctype
-                keytype = struct.unpack(__endian + 'h', stash.read(2))[0]
-                keylen = struct.unpack(__endian + 'h', stash.read(2))[0]
-                keydata = stash.read(keylen)
-                #check that we parsed the whole file, so no surprises
-                keyoffset = stash.tell()
-                stash.seek(0,2)
-                if stash.tell() != keyoffset:
-                    logging.critical("Unexpected unprocessed data in Stash file (processed %ld bytes, %ld left)." % (keyoffset, stash.tell() - keyoffset))
-            else:
-                keyversion = 1
-                keyprinctype = 1
-                keyprinc = [self.realm,"K","M"]
-                keylen = struct.unpack('i', stash.read(4))[0]
-                keydata = stash.read(keylen)
-        except os.error:
-            logging.critical("Failed to retrieve Master Key from Stash file: %s")
-        #encode it in the asn.1 attribute
-        MasterKey = univ.Sequence()
-        MasterKey.setComponentByPosition(0, univ.Integer(keytype))
-        MasterKey.setComponentByPosition(1, univ.OctetString(keydata))
-        krbMKey = univ.Sequence()
-        krbMKey.setComponentByPosition(0, univ.Integer(keyversion))
-        krbMKey.setComponentByPosition(1, MasterKey)
-        asn1key = pyasn1.codec.ber.encoder.encode(krbMKey)
-
-        dn = "cn="+self.realm+",cn=kerberos,"+self.suffix
-        #protect the master key by adding an appropriate deny rule along with the key
-        mod = [(ldap.MOD_ADD, 'aci', ipautil.template_str(KRBMKEY_DENY_ACI, self.sub_dict)),
-               (ldap.MOD_ADD, 'krbMKey', str(asn1key))]
-        try:
-            self.admin_conn.modify_s(dn, mod)
-        except ldap.TYPE_OR_VALUE_EXISTS, e:
-            logging.critical("failed to add master key to kerberos database\n")
-            raise e
-
     def __create_ds_keytab(self):
         ldap_principal = "ldap/" + self.fqdn + "@" + self.realm
         installutils.kadmin_addprinc(ldap_principal)
@@ -506,12 +407,6 @@ class KrbInstance(service.Service):
 
         self.move_service_to_host(host_principal)
 
-    def __export_kadmin_changepw_keytab(self):
-        installutils.kadmin_modprinc("kadmin/changepw", "+requires_preauth")
-
-        self.fstore.backup_file("/var/kerberos/krb5kdc/kpasswd.keytab")
-        installutils.create_keytab("/var/kerberos/krb5kdc/kpasswd.keytab", "kadmin/changepw")
-
     def __setup_pkinit(self):
         if self.self_signed_ca:
             ca_db = certs.CertDB(self.realm,
@@ -541,7 +436,7 @@ class KrbInstance(service.Service):
 
         # Create the special anonymous principal
         installutils.kadmin_addprinc(princ_realm)
-        dn = "krbprincipalname=%s,cn=%s,cn=kerberos,%s" % (princ_realm, self.realm, self.suffix)
+        dn = DN(('krbprincipalname', princ_realm), self.get_realm_suffix())
         self.admin_conn.inactivateEntry(dn, False)
 
     def __convert_to_gssapi_replication(self):
@@ -549,7 +444,7 @@ class KrbInstance(service.Service):
                                               self.fqdn,
                                               self.dm_password)
         repl.convert_to_gssapi_replication(self.master_fqdn,
-                                           r_binddn="cn=Directory Manager",
+                                           r_binddn=DN(('cn', 'Directory Manager')),
                                            r_bindpw=self.dm_password)
 
     def uninstall(self):
@@ -564,11 +459,11 @@ class KrbInstance(service.Service):
         except:
             pass
 
-        for f in ["/var/kerberos/krb5kdc/ldappwd", "/var/kerberos/krb5kdc/kdc.conf", "/etc/krb5.conf"]:
+        for f in ["/var/kerberos/krb5kdc/kdc.conf", "/etc/krb5.conf"]:
             try:
                 self.fstore.restore_file(f)
             except ValueError, error:
-                logging.debug(error)
+                root_logger.debug(error)
                 pass
 
         if not enabled is None and not enabled:
