@@ -17,43 +17,60 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import logging, sys
+import sys
 import os, socket
 import tempfile
+import pwd
 from ipapython import sysrestore
 from ipapython import ipautil
+from ipapython import dogtag
 from ipapython import services as ipaservices
 from ipalib import errors
+from ipapython.dn import DN
 import ldap
 from ipaserver import ipaldap
 import base64
 import time
 import datetime
 from ipaserver.install import installutils
+from ipapython.ipa_log_manager import *
 
 CACERT = "/etc/ipa/ca.crt"
 
+# Autobind modes
+AUTO = 1
+ENABLED = 2
+DISABLED = 3
+
+# The service name as stored in cn=masters,cn=ipa,cn=etc. In the tuple
+# the first value is the *nix service name, the second the start order.
 SERVICE_LIST = {
     'KDC':('krb5kdc', 10),
-    'KPASSWD':('ipa_kpasswd', 20),
+    'KPASSWD':('kadmin', 20),
     'DNS':('named', 30),
+    'MEMCACHE':('ipa_memcached', 39),
     'HTTP':('httpd', 40),
-    'CA':('pki-cad', 50)
+    'CA':('%sd' % dogtag.configured_constants().PKI_INSTANCE_NAME, 50),
+    'ADTRUST':('smb', 60),
+    'EXTID':('winbind', 70)
 }
 
 def print_msg(message, output_fd=sys.stdout):
-    logging.debug(message)
+    root_logger.debug(message)
     output_fd.write(message)
     output_fd.write("\n")
 
 
 class Service(object):
-    def __init__(self, service_name, sstore=None, dm_password=None):
+    def __init__(self, service_name, service_desc=None, sstore=None, dm_password=None, ldapi=True, autobind=AUTO):
         self.service_name = service_name
+        self.service_desc = service_desc
         self.service = ipaservices.service(service_name)
         self.steps = []
         self.output_fd = sys.stdout
         self.dm_password = dm_password
+        self.ldapi = ldapi
+        self.autobind = autobind
 
         self.fqdn = socket.gethostname()
         self.admin_conn = None
@@ -64,12 +81,49 @@ class Service(object):
             self.sstore = sysrestore.StateFile('/var/lib/ipa/sysrestore')
 
         self.realm = None
-        self.suffix = None
+        self.suffix = DN()
         self.principal = None
         self.dercert = None
 
     def ldap_connect(self):
-        self.admin_conn = self.__get_conn(self.fqdn, self.dm_password)
+        # If DM password is provided, we use it
+        # If autobind was requested, attempt autobind when root and ldapi
+        # If autobind was disabled or not succeeded, go with GSSAPI
+        # LDAPI can be used with either autobind or GSSAPI
+        # LDAPI requires realm to be set
+        try:
+            if self.ldapi:
+                if not self.realm:
+                    raise errors.NotFound(reason="realm is missing for %s" % (self))
+                conn = ipaldap.IPAdmin(ldapi=self.ldapi, realm=self.realm)
+            else:
+                conn = ipaldap.IPAdmin(self.fqdn, port=389)
+            if self.dm_password:
+                conn.do_simple_bind(bindpw=self.dm_password)
+            elif self.autobind in [AUTO, ENABLED]:
+                if os.getegid() == 0 and self.ldapi:
+                    try:
+                        # autobind
+                        pw_name = pwd.getpwuid(os.geteuid()).pw_name
+                        conn.do_external_bind(pw_name)
+                    except errors.NotFound, e:
+                        if self.autobind == AUTO:
+                            # Fall back
+                            conn.do_sasl_gssapi_bind()
+                        else:
+                            # autobind was required and failed, raise
+                            # exception that it failed
+                            raise e
+                else:
+                    conn.do_sasl_gssapi_bind()
+            else:
+                conn.do_sasl_gssapi_bind()
+        except Exception, e:
+            root_logger.debug("Could not connect to the Directory Server on %s: %s" % (self.fqdn, str(e)))
+            raise e
+
+        self.admin_conn = conn
+
 
     def ldap_disconnect(self):
         self.admin_conn.unbind()
@@ -80,7 +134,6 @@ class Service(object):
         pw_name = None
         fd = None
         path = ipautil.SHARE_DIR + ldif
-        hostname = installutils.get_fqdn()
         nologlist=[]
 
         if sub_dict is not None:
@@ -94,22 +147,32 @@ class Service(object):
             if sub_dict.has_key('RANDOM_PASSWORD'):
                 nologlist.append(sub_dict['RANDOM_PASSWORD'])
 
+        args = ["/usr/bin/ldapmodify", "-v", "-f", path]
+
+        # As we always connect to the local host,
+        # use URI of admin connection
+        if not self.admin_conn:
+            self.ldap_connect()
+        args += ["-H", self.admin_conn.uri]
+
+        auth_parms = []
         if self.dm_password:
             [pw_fd, pw_name] = tempfile.mkstemp()
             os.write(pw_fd, self.dm_password)
             os.close(pw_fd)
             auth_parms = ["-x", "-D", "cn=Directory Manager", "-y", pw_name]
         else:
-            auth_parms = ["-Y", "GSSAPI"]
+            # always try GSSAPI auth when not using DM password or not being root
+            if os.getegid() != 0:
+                auth_parms = ["-Y", "GSSAPI"]
 
-        args = ["/usr/bin/ldapmodify", "-h", hostname, "-v", "-f", path]
         args += auth_parms
 
         try:
             try:
                 ipautil.run(args, nolog=nologlist)
             except ipautil.CalledProcessError, e:
-                logging.critical("Failed to load %s: %s" % (ldif, str(e)))
+                root_logger.critical("Failed to load %s: %s" % (ldif, str(e)))
         finally:
             if pw_name:
                 os.remove(pw_name)
@@ -123,15 +186,15 @@ class Service(object):
         cn=kerberos to cn=services
         """
 
-        dn = "krbprincipalname=%s,cn=%s,cn=kerberos,%s" % (principal, self.realm, self.suffix)
+        dn = DN(('krbprincipalname', principal), ('cn', self.realm), ('cn', 'kerberos'), self.suffix)
         try:
             entry = self.admin_conn.getEntry(dn, ldap.SCOPE_BASE)
         except errors.NotFound:
             # There is no service in the wrong location, nothing to do.
             # This can happen when installing a replica
-            return
-        newdn = "krbprincipalname=%s,cn=services,cn=accounts,%s" % (principal, self.suffix)
-        hostdn = "fqdn=%s,cn=computers,cn=accounts,%s" % (self.fqdn, self.suffix)
+            return None
+        newdn = DN(('krbprincipalname', principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
+        hostdn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
         self.admin_conn.deleteEntry(dn)
         entry.dn = newdn
         classes = entry.getValues("objectclass")
@@ -151,8 +214,8 @@ class Service(object):
         if not self.admin_conn:
             self.ldap_connect()
 
-        dn = "krbprincipalname=%s,cn=services,cn=accounts,%s" % (principal, self.suffix)
-        hostdn = "fqdn=%s,cn=computers,cn=accounts,%s" % (self.fqdn, self.suffix)
+        dn = DN(('krbprincipalname', principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
+        hostdn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
         entry = ipaldap.Entry(dn)
         entry.setValues("objectclass", ["krbprincipal", "krbprincipalaux", "krbticketpolicyaux", "ipaobject", "ipaservice", "pkiuser"])
         entry.setValue("krbprincipalname", principal)
@@ -168,15 +231,26 @@ class Service(object):
         This server cert should be in DER format.
         """
 
-        if not self.admin_conn:
-            self.ldap_connect()
+        # add_cert_to_service() is relatively rare operation
+        # we actually call it twice during ipa-server-install, for different
+        # instances: ds and cs. Unfortunately, it may happen that admin
+        # connection was created well before add_cert_to_service() is called
+        # If there are other operations in between, it will become stale and
+        # since we are using SimpleLDAPObject, not ReconnectLDAPObject, the
+        # action will fail. Thus, explicitly disconnect and connect again.
+        # Using ReconnectLDAPObject instead of SimpleLDAPObject was considered
+        # but consequences for other parts of the framework are largely
+        # unknown.
+        if self.admin_conn:
+            self.ldap_disconnect()
+        self.ldap_connect()
 
-        dn = "krbprincipalname=%s,cn=services,cn=accounts,%s" % (self.principal, self.suffix)
+        dn = DN(('krbprincipalname', self.principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
         mod = [(ldap.MOD_ADD, 'userCertificate', self.dercert)]
         try:
             self.admin_conn.modify_s(dn, mod)
         except Exception, e:
-            logging.critical("Could not add certificate to service %s entry: %s" % (self.principal, str(e)))
+            root_logger.critical("Could not add certificate to service %s entry: %s" % (self.principal, str(e)))
 
     def is_configured(self):
         return self.sstore.has_state(self.service_name)
@@ -187,11 +261,11 @@ class Service(object):
     def stop(self, instance_name="", capture_output=True):
         self.service.stop(instance_name, capture_output=capture_output)
 
-    def start(self, instance_name="", capture_output=True):
-        self.service.start(instance_name, capture_output=capture_output)
+    def start(self, instance_name="", capture_output=True, wait=True):
+        self.service.start(instance_name, capture_output=capture_output, wait=wait)
 
-    def restart(self, instance_name="", capture_output=True):
-        self.service.restart(instance_name, capture_output=capture_output)
+    def restart(self, instance_name="", capture_output=True, wait=True):
+        self.service.restart(instance_name, capture_output=capture_output, wait=wait)
 
     def is_running(self):
         return self.service.is_running()
@@ -223,7 +297,43 @@ class Service(object):
     def step(self, message, method):
         self.steps.append((message, method))
 
-    def start_creation(self, message, runtime=-1):
+    def start_creation(self, start_message=None, end_message=None,
+        show_service_name=True, runtime=-1):
+        """
+        Starts creation of the service.
+
+        Use start_message and end_message for explicit messages
+        at the beggining / end of the process. Otherwise they are generated
+        using the service description (or service name, if the description has
+        not been provided).
+
+        Use show_service_name to include service name in generated descriptions.
+        """
+
+        if start_message is None:
+            # no other info than mandatory service_name provided, use that
+            if self.service_desc is None:
+                start_message = "Configuring %s" % self.service_name
+
+            # description should be more accurate than service name
+            else:
+                start_message = "Configuring %s" % self.service_desc
+                if show_service_name:
+                    start_message = "%s (%s)" % (start_message, self.service_name)
+
+        if end_message is None:
+            if self.service_desc is None:
+                if show_service_name:
+                    end_message = "Done configuring %s." % self.service_name
+                else:
+                    end_message = "Done."
+            else:
+                if show_service_name:
+                    end_message = "Done configuring %s (%s)." % (
+                        self.service_desc, self.service_name)
+                else:
+                    end_message = "Done configuring %s." % self.service_desc
+
         if runtime > 0:
             plural=''
             est = time.localtime(runtime)
@@ -231,15 +341,15 @@ class Service(object):
                 if est.tm_min > 1:
                     plural = 's'
                 if est.tm_sec > 0:
-                    self.print_msg('%s: Estimated time %d minute%s %d seconds' % (message, est.tm_min, plural, est.tm_sec))
+                    self.print_msg('%s: Estimated time %d minute%s %d seconds' % (start_message, est.tm_min, plural, est.tm_sec))
                 else:
-                    self.print_msg('%s: Estimated time %d minute%s' % (message, est.tm_min, plural))
+                    self.print_msg('%s: Estimated time %d minute%s' % (start_message, est.tm_min, plural))
             else:
                 if est.tm_sec > 1:
                     plural = 's'
-                self.print_msg('%s: Estimated time %d second%s' % (message, est.tm_sec, plural))
+                self.print_msg('%s: Estimated time %d second%s' % (start_message, est.tm_sec, plural))
         else:
-            self.print_msg(message)
+            self.print_msg(start_message)
 
         step = 0
         for (message, method) in self.steps:
@@ -248,36 +358,20 @@ class Service(object):
             method()
             e = datetime.datetime.now()
             d = e - s
-            logging.debug("  duration: %d seconds" % d.seconds)
+            root_logger.debug("  duration: %d seconds" % d.seconds)
             step += 1
 
-        self.print_msg("done configuring %s." % self.service_name)
+        self.print_msg(end_message)
 
         self.steps = []
 
-    def __get_conn(self, fqdn, dm_password):
-        # If we are passed a password we'll use it as the DM password
-        # otherwise we'll do a GSSAPI bind.
-        try:
-#            conn = ipaldap.IPAdmin(fqdn, port=636, cacert=CACERT)
-            conn = ipaldap.IPAdmin(fqdn, port=389)
-            if dm_password:
-                conn.do_simple_bind(bindpw=dm_password)
-            else:
-                conn.do_sasl_gssapi_bind()
-        except Exception, e:
-            logging.debug("Could not connect to the Directory Server on %s: %s" % (fqdn, str(e)))
-            raise e
-
-        return conn
-
     def ldap_enable(self, name, fqdn, dm_password, ldap_suffix):
+        assert isinstance(ldap_suffix, DN)
         self.disable()
-        conn = self.__get_conn(fqdn, dm_password)
+        if not self.admin_conn:
+            self.ldap_connect()
 
-        entry_name = "cn=%s,cn=%s,%s,%s" % (name, fqdn,
-                                            "cn=masters,cn=ipa,cn=etc",
-                                            ldap_suffix)
+        entry_name = DN(('cn', name), ('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), ldap_suffix)
         order = SERVICE_LIST[name][1]
         entry = ipaldap.Entry(entry_name)
         entry.setValues("objectclass",
@@ -287,22 +381,27 @@ class Service(object):
                         "enabledService", "startOrder " + str(order))
 
         try:
-            conn.add_s(entry)
-        except ldap.ALREADY_EXISTS, e:
-            logging.critical("failed to add %s Service startup entry" % name)
+            self.admin_conn.addEntry(entry)
+        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry), e:
+            root_logger.debug("failed to add %s Service startup entry" % name)
             raise e
 
 class SimpleServiceInstance(Service):
-    def create_instance(self, gensvc_name=None, fqdn=None, dm_password=None, ldap_suffix=None):
+    def create_instance(self, gensvc_name=None, fqdn=None, dm_password=None, ldap_suffix=None, realm=None):
         self.gensvc_name = gensvc_name
         self.fqdn = fqdn
         self.dm_password = dm_password
         self.suffix = ldap_suffix
+        self.realm = realm
+        if not realm:
+            self.ldapi = False
 
         self.step("starting %s " % self.service_name, self.__start)
         self.step("configuring %s to start on boot" % self.service_name, self.__enable)
         self.start_creation("Configuring %s" % self.service_name)
 
+    suffix = ipautil.dn_attribute_property('_ldap_suffix')
+
     def __start(self):
         self.backup_state("running", self.is_running())
         self.restart()
