@@ -20,6 +20,8 @@
 Test the `ipalib/plugins/hbacrule.py` module.
 """
 
+from nose.tools import raises, assert_raises  # pylint: disable=E0611
+
 from xmlrpc_test import XMLRPC_test, assert_attr_equal
 from ipalib import api
 from ipalib import errors
@@ -41,13 +43,15 @@ class test_hbac(XMLRPC_test):
 
     test_user = u'hbacrule_test_user'
     test_group = u'hbacrule_test_group'
-    test_host = u'hbacrule.test-netgroup'
+    test_host = u'hbacrule.testnetgroup'
     test_hostgroup = u'hbacrule_test_hostgroup'
-    test_sourcehost = u'hbacrule.test-src-host'
+    test_sourcehost = u'hbacrule.testsrchost'
     test_sourcehostgroup = u'hbacrule_test_src_hostgroup'
     test_service = u'sshd'
     test_host_external = u'notfound.example.com'
 
+    test_invalid_sourcehost = u'inv+alid#srchost.nonexist.com'
+
     def test_0_hbacrule_add(self):
         """
         Test adding a new HBAC rule using `xmlrpc.hbacrule_add`.
@@ -63,18 +67,14 @@ class test_hbac(XMLRPC_test):
         assert_attr_equal(entry, 'ipaenabledflag', 'TRUE')
         assert_attr_equal(entry, 'description', self.rule_desc)
 
+    @raises(errors.DuplicateEntry)
     def test_1_hbacrule_add(self):
         """
         Test adding an existing HBAC rule using `xmlrpc.hbacrule_add'.
         """
-        try:
-            api.Command['hbacrule_add'](
+        api.Command['hbacrule_add'](
                 self.rule_name, accessruletype=self.rule_type
             )
-        except errors.DuplicateEntry:
-            pass
-        else:
-            assert False
 
     def test_2_hbacrule_show(self):
         """
@@ -125,7 +125,7 @@ class test_hbac(XMLRPC_test):
         Test searching for HBAC rules using `xmlrpc.hbacrule_find`.
         """
         ret = api.Command['hbacrule_find'](
-            name=self.rule_name, accessruletype=self.rule_type,
+            cn=self.rule_name, accessruletype=self.rule_type,
             description=self.rule_desc_mod
         )
         assert ret['truncated'] is False
@@ -157,7 +157,7 @@ class test_hbac(XMLRPC_test):
             self.test_sourcehostgroup, description=u'desc'
         )
         self.failsafe_add(api.Object.hbacsvc,
-            self.test_service, description=u'desc', force=True
+            self.test_service, description=u'desc',
         )
 
     def test_8_hbacrule_add_user(self):
@@ -265,8 +265,8 @@ class test_hbac(XMLRPC_test):
         assert 'hostgroup' in failed['memberhost']
         assert not failed['memberhost']['hostgroup']
         entry = ret['result']
-        assert 'memberhost_host' not in res[1]
-        assert 'memberhost_hostgroup' not in res[1]
+        assert 'memberhost_host' not in entry
+        assert 'memberhost_hostgroup' not in entry
 
     def test_a_hbacrule_add_sourcehost(self):
         """
@@ -286,6 +286,17 @@ class test_hbac(XMLRPC_test):
         assert_attr_equal(entry, 'sourcehost_host', self.test_host)
         assert_attr_equal(entry, 'sourcehost_hostgroup', self.test_hostgroup)
 
+    def test_a_hbacrule_add_invalid_sourcehost(self):
+        """
+        Test adding invalid source host to HBAC rule using `xmlrpc.hbacrule_add_host`.
+        """
+        try:
+            api.Command['hbacrule_add_sourcehost'](
+                self.rule_name, host=self.test_invalid_sourcehost, hostgroup=self.test_hostgroup
+            )
+        except errors.ValidationError:
+            pass
+
     def test_a_hbacrule_add_service(self):
         """
         Test adding service to HBAC rule using `xmlrpc.hbacrule_add_service`.
@@ -316,7 +327,7 @@ class test_hbac(XMLRPC_test):
         entry = ret['result']
         assert 'memberservice service' not in entry
 
-    def test_b_hbacrule_remove_host(self):
+    def test_b_hbacrule_remove_sourcehost(self):
         """
         Test removing source host and hostgroup from HBAC rule using `xmlrpc.hbacrule_remove_host`.
         """
@@ -366,6 +377,15 @@ class test_hbac(XMLRPC_test):
         entry = ret['result']
         assert_attr_equal(entry, 'externalhost', self.test_host_external)
 
+    @raises(errors.ValidationError)
+    def test_c_hbacrule_mod_invalid_external_setattr(self):
+        """
+        Test adding the same external host using `xmlrpc.hbacrule_add_host`.
+        """
+        ret = api.Command['hbacrule_mod'](
+            self.rule_name, setattr=self.test_invalid_sourcehost
+        )
+
     def test_c_hbacrule_remove_external_host(self):
         """
         Test removing external source host using `xmlrpc.hbacrule_remove_host`.
@@ -400,20 +420,6 @@ class test_hbac(XMLRPC_test):
         assert not failed['sourcehost']['hostgroup']
         entry = ret['result']
 
-    def test_c_hbacrule_zap_testing_data(self):
-        """
-        Clear data for HBAC plugin testing.
-        """
-        api.Command['hbacrule_remove_host'](self.rule_name, host=self.test_host)
-        api.Command['hbacrule_remove_host'](self.rule_name, hostgroup=self.test_hostgroup)
-        api.Command['user_del'](self.test_user)
-        api.Command['group_del'](self.test_group)
-        api.Command['host_del'](self.test_host)
-        api.Command['hostgroup_del'](self.test_hostgroup)
-        api.Command['host_del'](self.test_sourcehost)
-        api.Command['hostgroup_del'](self.test_sourcehostgroup)
-        api.Command['hbacsvc_del'](self.test_service)
-
     def test_d_hbacrule_disable(self):
         """
         Test disabling HBAC rule using `xmlrpc.hbacrule_disable`.
@@ -433,29 +439,50 @@ class test_hbac(XMLRPC_test):
          # FIXME: Should this be 'enabled' or 'TRUE'?
         assert_attr_equal(entry, 'ipaenabledflag', 'TRUE')
 
+    def test_ea_hbacrule_disable_setattr(self):
+        """
+        Test disabling HBAC rule using setattr
+        """
+        command_result = api.Command['hbacrule_mod'](
+            self.rule_name, setattr=u'ipaenabledflag=false')
+        assert command_result['result']['ipaenabledflag'] == (u'FALSE',)
+        entry = api.Command['hbacrule_show'](self.rule_name)['result']
+        assert_attr_equal(entry, 'ipaenabledflag', 'FALSE')
+
+    def test_eb_hbacrule_enable_setattr(self):
+        """
+        Test enabling HBAC rule using setattr
+        """
+        command_result = api.Command['hbacrule_mod'](
+            self.rule_name, setattr=u'ipaenabledflag=1')
+        assert command_result['result']['ipaenabledflag'] == (u'TRUE',)
+        # check it's really enabled
+        entry = api.Command['hbacrule_show'](self.rule_name)['result']
+        assert_attr_equal(entry, 'ipaenabledflag', 'TRUE')
+
+    @raises(errors.MutuallyExclusiveError)
     def test_f_hbacrule_exclusiveuser(self):
         """
         Test adding a user to an HBAC rule when usercat='all'
         """
         api.Command['hbacrule_mod'](self.rule_name, usercategory=u'all')
         try:
-            api.Command['hbacrule_add_user'](self.rule_name, users='admin')
-        except errors.MutuallyExclusiveError:
-            pass
-        api.Command['hbacrule_mod'](self.rule_name, usercategory=u'')
+            api.Command['hbacrule_add_user'](self.rule_name, user=u'admin')
+        finally:
+            api.Command['hbacrule_mod'](self.rule_name, usercategory=u'')
 
+    @raises(errors.MutuallyExclusiveError)
     def test_g_hbacrule_exclusiveuser(self):
         """
         Test setting usercat='all' in an HBAC rule when there are users
         """
-        api.Command['hbacrule_add_user'](self.rule_name, users='admin')
+        api.Command['hbacrule_add_user'](self.rule_name, user=u'admin')
         try:
             api.Command['hbacrule_mod'](self.rule_name, usercategory=u'all')
-        except errors.MutuallyExclusiveError:
-            pass
         finally:
-            api.Command['hbacrule_remove_user'](self.rule_name, users='admin')
+            api.Command['hbacrule_remove_user'](self.rule_name, user=u'admin')
 
+    @raises(errors.MutuallyExclusiveError)
     def test_h_hbacrule_exclusivehost(self):
         """
         Test adding a host to an HBAC rule when hostcat='all'
@@ -463,10 +490,10 @@ class test_hbac(XMLRPC_test):
         api.Command['hbacrule_mod'](self.rule_name, hostcategory=u'all')
         try:
             api.Command['hbacrule_add_host'](self.rule_name, host=self.test_host)
-        except errors.MutuallyExclusiveError:
-            pass
-        api.Command['hbacrule_mod'](self.rule_name, hostcategory=u'')
+        finally:
+            api.Command['hbacrule_mod'](self.rule_name, hostcategory=u'')
 
+    @raises(errors.MutuallyExclusiveError)
     def test_i_hbacrule_exclusivehost(self):
         """
         Test setting hostcat='all' in an HBAC rule when there are hosts
@@ -474,22 +501,21 @@ class test_hbac(XMLRPC_test):
         api.Command['hbacrule_add_host'](self.rule_name, host=self.test_host)
         try:
             api.Command['hbacrule_mod'](self.rule_name, hostcategory=u'all')
-        except errors.MutuallyExclusiveError:
-            pass
         finally:
             api.Command['hbacrule_remove_host'](self.rule_name, host=self.test_host)
 
+    @raises(errors.MutuallyExclusiveError)
     def test_j_hbacrule_exclusiveservice(self):
         """
         Test adding a service to an HBAC rule when servicecat='all'
         """
         api.Command['hbacrule_mod'](self.rule_name, servicecategory=u'all')
         try:
-            api.Command['hbacrule_add_host'](self.rule_name, hbacsvc=self.test_service)
-        except errors.MutuallyExclusiveError:
-            pass
-        api.Command['hbacrule_mod'](self.rule_name, servicecategory=u'')
+            api.Command['hbacrule_add_service'](self.rule_name, hbacsvc=self.test_service)
+        finally:
+            api.Command['hbacrule_mod'](self.rule_name, servicecategory=u'')
 
+    @raises(errors.MutuallyExclusiveError)
     def test_k_hbacrule_exclusiveservice(self):
         """
         Test setting servicecat='all' in an HBAC rule when there are services
@@ -497,35 +523,70 @@ class test_hbac(XMLRPC_test):
         api.Command['hbacrule_add_service'](self.rule_name, hbacsvc=self.test_service)
         try:
             api.Command['hbacrule_mod'](self.rule_name, servicecategory=u'all')
-        except errors.MutuallyExclusiveError:
-            pass
         finally:
             api.Command['hbacrule_remove_service'](self.rule_name, hbacsvc=self.test_service)
 
+    @raises(errors.ValidationError)
     def test_l_hbacrule_add(self):
         """
         Test adding a new HBAC rule with a deny type.
         """
-        try:
-            api.Command['hbacrule_add'](
-                u'denyrule',
-                accessruletype=u'deny',
-                description=self.rule_desc,
-            )
-        except errors.ValidationError:
-            pass
+        api.Command['hbacrule_add'](
+            u'denyrule',
+            accessruletype=u'deny',
+            description=self.rule_desc,
+        )
 
+    @raises(errors.ValidationError)
     def test_m_hbacrule_add(self):
         """
         Test changing an HBAC rule to the deny type
         """
-        try:
-            api.Command['hbacrule_mod'](
-                self.rule_name,
-                accessruletype=u'deny',
-            )
-        except errors.ValidationError:
-            pass
+        api.Command['hbacrule_mod'](
+            self.rule_name,
+            accessruletype=u'deny',
+        )
+
+    def test_n_hbacrule_links(self):
+        """
+        Test adding various links to HBAC rule
+        """
+        api.Command['hbacrule_add_sourcehost'](
+            self.rule_name, host=self.test_host, hostgroup=self.test_hostgroup
+        )
+        api.Command['hbacrule_add_service'](
+            self.rule_name, hbacsvc=self.test_service
+        )
+
+        entry = api.Command['hbacrule_show'](self.rule_name)['result']
+        assert_attr_equal(entry, 'cn', self.rule_name)
+        assert_attr_equal(entry, 'sourcehost_host', self.test_host)
+        assert_attr_equal(entry, 'sourcehost_hostgroup', self.test_hostgroup)
+        assert_attr_equal(entry, 'memberservice_hbacsvc', self.test_service)
+
+    def test_y_hbacrule_zap_testing_data(self):
+        """
+        Clear data for HBAC plugin testing.
+        """
+        api.Command['hbacrule_remove_host'](self.rule_name, host=self.test_host)
+        api.Command['hbacrule_remove_host'](self.rule_name, hostgroup=self.test_hostgroup)
+        api.Command['user_del'](self.test_user)
+        api.Command['group_del'](self.test_group)
+        api.Command['host_del'](self.test_host)
+        api.Command['hostgroup_del'](self.test_hostgroup)
+        api.Command['host_del'](self.test_sourcehost)
+        api.Command['hostgroup_del'](self.test_sourcehostgroup)
+        api.Command['hbacsvc_del'](self.test_service)
+
+    def test_k_2_sudorule_referential_integrity(self):
+        """
+        Test that links in HBAC rule were removed by referential integrity plugin
+        """
+        entry = api.Command['hbacrule_show'](self.rule_name)['result']
+        assert_attr_equal(entry, 'cn', self.rule_name)
+        assert 'sourcehost_host' not in entry
+        assert 'sourcehost_hostgroup' not in entry
+        assert 'memberservice_hbacsvc' not in entry
 
     def test_z_hbacrule_del(self):
         """
@@ -533,9 +594,5 @@ class test_hbac(XMLRPC_test):
         """
         api.Command['hbacrule_del'](self.rule_name)
         # verify that it's gone
-        try:
+        with assert_raises(errors.NotFound):
             api.Command['hbacrule_show'](self.rule_name)
-        except errors.NotFound:
-            pass
-        else:
-            assert False
