@@ -22,10 +22,16 @@
 Test the `ipalib.plugins.host` module.
 """
 
+import os
+import tempfile
+from ipapython import ipautil
 from ipalib import api, errors, x509
-from ipalib.dn import *
-from tests.test_xmlrpc.xmlrpc_test import Declarative, fuzzy_uuid, fuzzy_digits
-from tests.test_xmlrpc.xmlrpc_test import fuzzy_hash, fuzzy_date, fuzzy_issuer
+from ipapython.dn import DN
+from nose.tools import raises, assert_raises
+from nose.plugins.skip import Skip, SkipTest
+from tests.test_xmlrpc.xmlrpc_test import (Declarative, XMLRPC_test,
+    fuzzy_uuid, fuzzy_digits, fuzzy_hash, fuzzy_date, fuzzy_issuer,
+    fuzzy_hex)
 from tests.test_xmlrpc import objectclasses
 import base64
 
@@ -47,6 +53,7 @@ dn3 = DN(('fqdn',fqdn3),('cn','computers'),('cn','accounts'),
 fqdn4 = u'testhost2.lab.%s' % api.env.domain
 dn4 = DN(('fqdn',fqdn4),('cn','computers'),('cn','accounts'),
          api.env.basedn)
+invalidfqdn1 = u'foo_bar.lab.%s' % api.env.domain
 
 # We can use the same cert we generated for the service tests
 fd = open('tests/test_xmlrpc/service.crt', 'r')
@@ -55,6 +62,9 @@ servercert = ''.join(servercert)
 servercert = x509.strip_header(servercert)
 fd.close()
 
+sshpubkey = u'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDGAX3xAeLeaJggwTqMjxNwa6XHBUAikXPGMzEpVrlLDCZtv00djsFTBi38PkgxBJVkgRWMrcBsr/35lq7P6w8KGIwA8GI48Z0qBS2NBMJ2u9WQ2hjLN6GdMlo77O0uJY3251p12pCVIS/bHRSq8kHO2No8g7KA9fGGcagPfQH+ee3t7HUkpbQkFTmbPPN++r3V8oVUk5LxbryB3UIIVzNmcSIn3JrXynlvui4MixvrtX6zx+O/bBo68o8/eZD26QrahVbA09fivrn/4h3TM019Eu/c2jOdckfU3cHUV/3Tno5d6JicibyaoDDK7S/yjdn5jhaz8MSEayQvFkZkiF0L public key test'
+sshpubkeyfp = u'13:67:6B:BF:4E:A2:05:8E:AE:25:8B:A1:31:DE:6F:1B public key test (ssh-rsa)'
+
 class test_host(Declarative):
 
     cleanup_commands = [
@@ -70,21 +80,24 @@ class test_host(Declarative):
         dict(
             desc='Try to retrieve non-existent %r' % fqdn1,
             command=('host_show', [fqdn1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % fqdn1),
         ),
 
 
         dict(
             desc='Try to update non-existent %r' % fqdn1,
             command=('host_mod', [fqdn1], dict(description=u'Nope')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % fqdn1),
         ),
 
 
         dict(
             desc='Try to delete non-existent %r' % fqdn1,
             command=('host_del', [fqdn1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % fqdn1),
         ),
 
 
@@ -101,7 +114,7 @@ class test_host(Declarative):
                 value=fqdn1,
                 summary=u'Added host "%s"' % fqdn1,
                 result=dict(
-                    dn=lambda x: DN(x) == dn1,
+                    dn=dn1,
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
@@ -121,11 +134,12 @@ class test_host(Declarative):
             command=('host_add', [fqdn1],
                 dict(
                     description=u'Test host 1',
-                    localityname=u'Undisclosed location 1',
+                    l=u'Undisclosed location 1',
                     force=True,
                 ),
             ),
-            expected=errors.DuplicateEntry(),
+            expected=errors.DuplicateEntry(message=u'host with name ' +
+                u'"%s" already exists' %  fqdn1),
         ),
 
 
@@ -136,7 +150,7 @@ class test_host(Declarative):
                 value=fqdn1,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == dn1,
+                    dn=dn1,
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
@@ -156,7 +170,7 @@ class test_host(Declarative):
                 value=fqdn1,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == dn1,
+                    dn=dn1,
                     cn=[fqdn1],
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
@@ -187,7 +201,7 @@ class test_host(Declarative):
                 summary=u'1 host matched',
                 result=[
                     dict(
-                        dn=lambda x: DN(x) == dn1,
+                        dn=dn1,
                         fqdn=[fqdn1],
                         description=[u'Test host 1'],
                         l=[u'Undisclosed location 1'],
@@ -210,7 +224,7 @@ class test_host(Declarative):
                 summary=u'1 host matched',
                 result=[
                     dict(
-                        dn=lambda x: DN(x) == dn1,
+                        dn=dn1,
                         cn=[fqdn1],
                         fqdn=[fqdn1],
                         description=[u'Test host 1'],
@@ -249,9 +263,9 @@ class test_host(Declarative):
                     usercertificate=[base64.b64decode(servercert)],
                     valid_not_before=fuzzy_date,
                     valid_not_after=fuzzy_date,
-                    subject=lambda x: DN(x) == \
-                        DN(('CN',api.env.host),('O',api.env.realm)),
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
                     serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
                     md5_fingerprint=fuzzy_hash,
                     sha1_fingerprint=fuzzy_hash,
                     issuer=fuzzy_issuer,
@@ -269,7 +283,7 @@ class test_host(Declarative):
                 value=fqdn1,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == dn1,
+                    dn=dn1,
                     fqdn=[fqdn1],
                     description=[u'Updated host 1'],
                     l=[u'Undisclosed location 1'],
@@ -280,9 +294,9 @@ class test_host(Declarative):
                     usercertificate=[base64.b64decode(servercert)],
                     valid_not_before=fuzzy_date,
                     valid_not_after=fuzzy_date,
-                    subject=lambda x: DN(x) == \
-                        DN(('CN',api.env.host),('O',api.env.realm)),
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
                     serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
                     md5_fingerprint=fuzzy_hash,
                     sha1_fingerprint=fuzzy_hash,
                     issuer=fuzzy_issuer,
@@ -303,7 +317,7 @@ class test_host(Declarative):
                 value=fqdn3,
                 summary=u'Added host "%s"' % fqdn3,
                 result=dict(
-                    dn=lambda x: DN(x) == dn3,
+                    dn=dn3,
                     fqdn=[fqdn3],
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
@@ -331,7 +345,7 @@ class test_host(Declarative):
                 value=fqdn4,
                 summary=u'Added host "%s"' % fqdn4,
                 result=dict(
-                    dn=lambda x: DN(x) == dn4,
+                    dn=dn4,
                     fqdn=[fqdn4],
                     description=[u'Test host 4'],
                     l=[u'Undisclosed location 4'],
@@ -361,7 +375,7 @@ class test_host(Declarative):
                     ),
                 ),
                 result=dict(
-                    dn=lambda x: DN(x) == dn3,
+                    dn=dn3,
                     fqdn=[fqdn3],
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
@@ -378,7 +392,7 @@ class test_host(Declarative):
                 value=fqdn3,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == dn3,
+                    dn=dn3,
                     fqdn=[fqdn3],
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
@@ -390,6 +404,39 @@ class test_host(Declarative):
             ),
         ),
 
+        dict(
+            desc='Search for hosts with --man-hosts and --not-man-hosts',
+            command=('host_find', [], {'man_host' : fqdn3, 'not_man_host' : fqdn1}),
+            expected=dict(
+                count=1,
+                truncated=False,
+                summary=u'1 host matched',
+                result=[
+                    dict(
+                        dn=dn3,
+                        fqdn=[fqdn3],
+                        description=[u'Test host 2'],
+                        l=[u'Undisclosed location 2'],
+                        krbprincipalname=[u'host/%s@%s' % (fqdn3, api.env.realm)],
+                        has_keytab=False,
+                        has_password=False,
+                        managedby_host=[u'%s' % fqdn3, u'%s' % fqdn1],
+                    ),
+                ],
+            ),
+        ),
+
+        dict(
+            desc='Try to search for hosts with --man-hosts',
+            command=('host_find', [], {'man_host' : [fqdn3,fqdn4]}),
+            expected=dict(
+                count=0,
+                truncated=False,
+                summary=u'0 hosts matched',
+                result=[],
+            ),
+        ),
+
         dict(
             desc='Remove managedby_host %r from %r' % (fqdn1, fqdn3),
             command=('host_remove_managedby', [fqdn3],
@@ -405,7 +452,7 @@ class test_host(Declarative):
                     ),
                 ),
                 result=dict(
-                    dn=lambda x: DN(x) == dn3,
+                    dn=dn3,
                     fqdn=[fqdn3],
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
@@ -425,11 +472,117 @@ class test_host(Declarative):
 
         dict(
             desc='Try to rename %r' % fqdn1,
-            command=('host_mod', [fqdn1], dict(setattr=u'fqdn=changed')),
+            command=('host_mod', [fqdn1], dict(setattr=u'fqdn=changed.example.com')),
             expected=errors.NotAllowedOnRDN()
         ),
 
 
+        dict(
+            desc='Add MAC address to %r' % fqdn1,
+            command=('host_mod', [fqdn1], dict(macaddress=u'00:50:56:30:F6:5F')),
+            expected=dict(
+                value=fqdn1,
+                summary=u'Modified host "%s"' % fqdn1,
+                result=dict(
+                    description=[u'Updated host 1'],
+                    fqdn=[fqdn1],
+                    l=[u'Undisclosed location 1'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    managedby_host=[u'%s' % fqdn1],
+                    usercertificate=[base64.b64decode(servercert)],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    macaddress=[u'00:50:56:30:F6:5F'],
+                    issuer=fuzzy_issuer,
+                    has_keytab=False,
+                    has_password=False,
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Add another MAC address to %r' % fqdn1,
+            command=('host_mod', [fqdn1], dict(macaddress=[u'00:50:56:30:F6:5F', u'00:50:56:2C:8D:82'])),
+            expected=dict(
+                value=fqdn1,
+                summary=u'Modified host "%s"' % fqdn1,
+                result=dict(
+                    description=[u'Updated host 1'],
+                    fqdn=[fqdn1],
+                    l=[u'Undisclosed location 1'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    managedby_host=[u'%s' % fqdn1],
+                    usercertificate=[base64.b64decode(servercert)],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    macaddress=[u'00:50:56:30:F6:5F', u'00:50:56:2C:8D:82'],
+                    issuer=fuzzy_issuer,
+                    has_keytab=False,
+                    has_password=False,
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Add an illegal MAC address to %r' % fqdn1,
+            command=('host_mod', [fqdn1], dict(macaddress=[u'xx'])),
+            expected=errors.ValidationError(name='macaddress',
+                error=u'Must be of the form HH:HH:HH:HH:HH:HH, where ' +
+                    u'each H is a hexadecimal character.'),
+        ),
+
+
+        dict(
+            desc='Add SSH public key to %r' % fqdn1,
+            command=('host_mod', [fqdn1], dict(ipasshpubkey=[sshpubkey])),
+            expected=dict(
+                value=fqdn1,
+                summary=u'Modified host "%s"' % fqdn1,
+                result=dict(
+                    description=[u'Updated host 1'],
+                    fqdn=[fqdn1],
+                    l=[u'Undisclosed location 1'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn1, api.env.realm)],
+                    managedby_host=[u'%s' % fqdn1],
+                    usercertificate=[base64.b64decode(servercert)],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    issuer=fuzzy_issuer,
+                    macaddress=[u'00:50:56:30:F6:5F', u'00:50:56:2C:8D:82'],
+                    ipasshpubkey=[sshpubkey],
+                    sshpubkeyfp=[sshpubkeyfp],
+                    has_keytab=False,
+                    has_password=False,
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Add an illegal SSH public key to %r' % fqdn1,
+            command=('host_mod', [fqdn1], dict(ipasshpubkey=[u'no-pty %s' % sshpubkey])),
+            expected=errors.ValidationError(name='sshpubkey',
+                error=u'options are not allowed'),
+        ),
+
+
         dict(
             desc='Delete %r' % fqdn1,
             command=('host_del', [fqdn1], {}),
@@ -444,21 +597,21 @@ class test_host(Declarative):
         dict(
             desc='Try to retrieve non-existent %r' % fqdn1,
             command=('host_show', [fqdn1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(reason=u'%s: host not found' % fqdn1),
         ),
 
 
         dict(
             desc='Try to update non-existent %r' % fqdn1,
             command=('host_mod', [fqdn1], dict(description=u'Nope')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(reason=u'%s: host not found' % fqdn1),
         ),
 
 
         dict(
             desc='Try to delete non-existent %r' % fqdn1,
             command=('host_del', [fqdn1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(reason=u'%s: host not found' % fqdn1),
         ),
 
         # Test deletion using a non-fully-qualified hostname. Services
@@ -476,7 +629,7 @@ class test_host(Declarative):
                 value=fqdn1,
                 summary=u'Added host "%s"' % fqdn1,
                 result=dict(
-                    dn=lambda x: DN(x) == dn1,
+                    dn=dn1,
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
@@ -497,7 +650,7 @@ class test_host(Declarative):
                 value=service1,
                 summary=u'Added service "%s"' % service1,
                 result=dict(
-                    dn=lambda x: DN(x) == service1dn,
+                    dn=service1dn,
                     krbprincipalname=[service1],
                     objectclass=objectclasses.service,
                     managedby_host=[fqdn1],
@@ -532,7 +685,8 @@ class test_host(Declarative):
         dict(
             desc='Try to add host not in DNS %r without force' % fqdn2,
             command=('host_add', [fqdn2], {}),
-            expected=errors.DNSNotARecordError(reason='Host does not have corresponding DNS A record'),
+            expected=errors.DNSNotARecordError(
+                reason=u'Host does not have corresponding DNS A record'),
         ),
 
 
@@ -549,18 +703,210 @@ class test_host(Declarative):
                 value=fqdn2,
                 summary=u'Added host "%s"' % fqdn2,
                 result=dict(
-                    dn=lambda x: DN(x) == dn2,
+                    dn=dn2,
                     fqdn=[fqdn2],
                     description=[u'Test host 2'],
                     l=[u'Undisclosed location 2'],
                     krbprincipalname=[u'host/%s@%s' % (fqdn2, api.env.realm)],
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
-                    managedby_host=[u'%s' % fqdn2],
+                    managedby_host=[fqdn2],
                     has_keytab=False,
                     has_password=False,
                 ),
             ),
         ),
 
+
+        # This test will only succeed when running against lite-server.py
+        # on same box as IPA install.
+        dict(
+            desc='Delete the current host (master?) %s should be caught' % api.env.host,
+            command=('host_del', [api.env.host], {}),
+            expected=errors.ValidationError(name='hostname',
+                error=u'An IPA master host cannot be deleted or disabled'),
+        ),
+
+
+        dict(
+            desc='Disable the current host (master?) %s should be caught' % api.env.host,
+            command=('host_disable', [api.env.host], {}),
+            expected=errors.ValidationError(name='hostname',
+                error=u'An IPA master host cannot be deleted or disabled'),
+        ),
+
+
+        dict(
+            desc='Test that validation is enabled on adds',
+            command=('host_add', [invalidfqdn1], {}),
+            expected=errors.ValidationError(name='hostname',
+                error=u'invalid domain-name: only letters, numbers, and - ' +
+                    u'are allowed. DNS label may not start or end with -'),
+        ),
+
+
+        # The assumption on these next 4 tests is that if we don't get a
+        # validation error then the request was processed normally.
+        dict(
+            desc='Test that validation is disabled on mods',
+            command=('host_mod', [invalidfqdn1], {}),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % invalidfqdn1),
+        ),
+
+
+        dict(
+            desc='Test that validation is disabled on deletes',
+            command=('host_del', [invalidfqdn1], {}),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % invalidfqdn1),
+        ),
+
+
+        dict(
+            desc='Test that validation is disabled on show',
+            command=('host_show', [invalidfqdn1], {}),
+            expected=errors.NotFound(
+                reason=u'%s: host not found' % invalidfqdn1),
+        ),
+
+
+        dict(
+            desc='Test that validation is disabled on find',
+            command=('host_find', [invalidfqdn1], {}),
+            expected=dict(
+                count=0,
+                truncated=False,
+                summary=u'0 hosts matched',
+                result=[],
+            ),
+        ),
+
+
+        dict(
+            desc='Add managedby_host %r to %r' % (fqdn3, fqdn4),
+            command=('host_add_managedby', [fqdn4], dict(host=fqdn3,),
+            ),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    managedby = dict(
+                        host=tuple(),
+                    ),
+                ),
+                result=dict(
+                    dn=dn4,
+                    fqdn=[fqdn4],
+                    description=[u'Test host 4'],
+                    l=[u'Undisclosed location 4'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn4, api.env.realm)],
+                    managedby_host=[fqdn4, fqdn3],
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Delete %r' % fqdn3,
+            command=('host_del', [fqdn3], {}),
+            expected=dict(
+                value=fqdn3,
+                summary=u'Deleted host "%s"' % fqdn3,
+                result=dict(failed=u''),
+            ),
+        ),
+
+
+        dict(
+            desc='Retrieve %r to verify that %r is gone from managedBy' % (fqdn4, fqdn3),
+            command=('host_show', [fqdn4], {}),
+            expected=dict(
+                value=fqdn4,
+                summary=None,
+                result=dict(
+                    dn=dn4,
+                    fqdn=[fqdn4],
+                    description=[u'Test host 4'],
+                    l=[u'Undisclosed location 4'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn4, api.env.realm)],
+                    has_keytab=False,
+                    has_password=False,
+                    managedby_host=[fqdn4],
+                ),
+            ),
+        ),
+
     ]
+
+class test_host_false_pwd_change(XMLRPC_test):
+
+    fqdn1 = u'testhost1.%s' % api.env.domain
+    short1 = u'testhost1'
+    new_pass = u'pass_123'
+    command = "ipa-client/ipa-join"
+
+    @classmethod
+    def setUpClass(cls):
+        [cls.keytabfd,cls.keytabname] = tempfile.mkstemp()
+        os.close(cls.keytabfd)
+
+        does_command_exist = os.path.isfile(cls.command)
+
+        if not does_command_exist:
+            raise SkipTest("Command '%s' not found" % cls.command)
+
+    # auxiliary function for checking whether the join operation has set
+    # correct attributes
+    def host_joined(self):
+        ret = api.Command['host_show'](self.fqdn1, all=True)
+        assert (ret['result']['has_keytab'] == True)
+        assert (ret['result']['has_password'] == False)
+
+    def test_a_join_host(self):
+        """
+        Create a test host and join him into IPA.
+        """
+
+        # create a test host with bulk enrollment password
+        random_pass = api.Command['host_add'](self.fqdn1, random=True, force=True)['result']['randompassword']
+
+        # joint the host with the bulk password
+        new_args = [self.command,
+                    "-s", api.env.host,
+                    "-h", self.fqdn1,
+                    "-k", self.keytabname,
+                    "-w", random_pass,
+                    "-q",
+                   ]
+        try:
+            # join operation may fail on 'adding key into keytab', but
+            # the keytab is not necessary for further tests
+            (out, err, rc) = ipautil.run(new_args, None)
+        except ipautil.CalledProcessError, e:
+            pass
+        finally:
+            self.host_joined()
+
+    @raises(errors.ValidationError)
+    def test_b_try_password(self):
+        """
+        Try to change the password of enrolled host with specified password
+        """
+        api.Command['host_mod'](self.fqdn1, userpassword=self.new_pass)
+
+    @raises(errors.ValidationError)
+    def test_c_try_random(self):
+        """
+        Try to change the password of enrolled host with random password
+        """
+        api.Command['host_mod'](self.fqdn1, random=True)
+
+    def test_d_cleanup(self):
+        """
+        Clean up test data
+        """
+        os.unlink(self.keytabname)
+        api.Command['host_del'](self.fqdn1)
+        # verify that it's gone
+        with assert_raises(errors.NotFound):
+            api.Command['host_show'](self.fqdn1)
