@@ -23,7 +23,7 @@ Test group nexting an indirect members
 from ipalib import api, errors
 from tests.test_xmlrpc import objectclasses
 from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
-from ipalib.dn import *
+from ipapython.dn import DN
 
 group1 = u'testgroup1'
 group2 = u'testgroup2'
@@ -80,9 +80,8 @@ class test_nesting(Declarative):
                     objectclass=objectclasses.group + [u'posixgroup'],
                     ipauniqueid=[fuzzy_uuid],
                     gidnumber=[fuzzy_digits],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup1'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup1'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -104,9 +103,8 @@ class test_nesting(Declarative):
                     gidnumber=[fuzzy_digits],
                     objectclass=objectclasses.group + [u'posixgroup'],
                     ipauniqueid=[fuzzy_uuid],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup2'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup2'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -126,9 +124,8 @@ class test_nesting(Declarative):
                     gidnumber=[fuzzy_digits],
                     objectclass=objectclasses.group + [u'posixgroup'],
                     ipauniqueid=[fuzzy_uuid],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup3'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup3'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -148,9 +145,8 @@ class test_nesting(Declarative):
                     gidnumber=[fuzzy_digits],
                     objectclass=objectclasses.group + [u'posixgroup'],
                     ipauniqueid=[fuzzy_uuid],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup4'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup4'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -175,22 +171,20 @@ class test_nesting(Declarative):
                     uid=[user1],
                     uidnumber=[fuzzy_digits],
                     gidnumber=[fuzzy_digits],
+                    mail=[u'%s@%s' % (user1, api.env.domain)],
                     displayname=[u'Test User1'],
                     cn=[u'Test User1'],
                     initials=[u'TU'],
                     ipauniqueid=[fuzzy_uuid],
-                    krbpwdpolicyreference=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn','global_policy'),('cn',api.env.realm),
-                            ('cn','kerberos'),api.env.basedn)],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',user1),('cn','groups'),('cn','accounts'),
-                            api.env.basedn)],
+                    krbpwdpolicyreference=[DN(('cn','global_policy'),('cn',api.env.realm),
+                                              ('cn','kerberos'),api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',user1),('cn','groups'),('cn','accounts'),
+                                        api.env.basedn)],
                     memberof_group=[u'ipausers'],
                     has_keytab=False,
                     has_password=False,
-                    dn=lambda x: DN(x) == \
-                        DN(('uid',user1),('cn','users'),('cn','accounts'),
-                           api.env.basedn)
+                    dn=DN(('uid',user1),('cn','users'),('cn','accounts'),
+                          api.env.basedn)
                 ),
             ),
         ),
@@ -215,22 +209,20 @@ class test_nesting(Declarative):
                     uid=[user2],
                     uidnumber=[fuzzy_digits],
                     gidnumber=[fuzzy_digits],
+                    mail=[u'%s@%s' % (user2, api.env.domain)],
                     displayname=[u'Test User2'],
                     cn=[u'Test User2'],
                     initials=[u'TU'],
                     ipauniqueid=[fuzzy_uuid],
-                    krbpwdpolicyreference=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn','global_policy'),('cn',api.env.realm),
-                            ('cn','kerberos'),api.env.basedn)],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',user2),('cn','groups'),('cn','accounts'),
-                            api.env.basedn)],
+                    krbpwdpolicyreference=[DN(('cn','global_policy'),('cn',api.env.realm),
+                                              ('cn','kerberos'),api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',user2),('cn','groups'),('cn','accounts'),
+                                        api.env.basedn)],
                     memberof_group=[u'ipausers'],
                     has_keytab=False,
                     has_password=False,
-                    dn=lambda x: DN(x) == \
-                        DN(('uid',user2),('cn','users'),('cn','accounts'),
-                           api.env.basedn)
+                    dn=DN(('uid',user2),('cn','users'),('cn','accounts'),
+                          api.env.basedn)
                 ),
             ),
         ),
@@ -255,22 +247,20 @@ class test_nesting(Declarative):
                     uid=[user3],
                     uidnumber=[fuzzy_digits],
                     gidnumber=[fuzzy_digits],
+                    mail=[u'%s@%s' % (user3, api.env.domain)],
                     displayname=[u'Test User3'],
                     cn=[u'Test User3'],
                     initials=[u'TU'],
                     ipauniqueid=[fuzzy_uuid],
-                    krbpwdpolicyreference=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn','global_policy'),('cn',api.env.realm),
-                            ('cn','kerberos'),api.env.basedn)],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',user3),('cn','groups'),('cn','accounts'),
-                            api.env.basedn)],
+                    krbpwdpolicyreference=[DN(('cn','global_policy'),('cn',api.env.realm),
+                                              ('cn','kerberos'),api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',user3),('cn','groups'),('cn','accounts'),
+                                        api.env.basedn)],
                     memberof_group=[u'ipausers'],
                     has_keytab=False,
                     has_password=False,
-                    dn=lambda x: DN(x) == \
-                        DN(('uid',user3),('cn','users'),('cn','accounts'),
-                           api.env.basedn)
+                    dn=DN(('uid',user3),('cn','users'),('cn','accounts'),
+                          api.env.basedn)
                 ),
             ),
         ),
@@ -295,22 +285,20 @@ class test_nesting(Declarative):
                     uid=[user4],
                     uidnumber=[fuzzy_digits],
                     gidnumber=[fuzzy_digits],
+                    mail=[u'%s@%s' % (user4, api.env.domain)],
                     displayname=[u'Test User4'],
                     cn=[u'Test User4'],
                     initials=[u'TU'],
                     ipauniqueid=[fuzzy_uuid],
-                    krbpwdpolicyreference=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn','global_policy'),('cn',api.env.realm),
-                            ('cn','kerberos'),api.env.basedn)],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',user4),('cn','groups'),('cn','accounts'),
-                            api.env.basedn)],
+                    krbpwdpolicyreference=[DN(('cn','global_policy'),('cn',api.env.realm),
+                                              ('cn','kerberos'),api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',user4),('cn','groups'),('cn','accounts'),
+                                        api.env.basedn)],
                     memberof_group=[u'ipausers'],
                     has_keytab=False,
                     has_password=False,
-                    dn=lambda x: DN(x) == \
-                        DN(('uid',user4),('cn','users'),('cn','accounts'),
-                           api.env.basedn)
+                    dn=DN(('uid',user4),('cn','users'),('cn','accounts'),
+                          api.env.basedn)
                 ),
             ),
         ),
@@ -375,9 +363,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group1),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group1),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_group': (group2,),
                         'gidnumber': [fuzzy_digits],
                         'cn': [group1],
@@ -401,9 +388,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group1),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group1),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_group': [group2, group3,],
                         'gidnumber': [fuzzy_digits],
                         'cn': [group1],
@@ -427,9 +413,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group2),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group2),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': (u'tuser1',),
                         'memberof_group': (u'testgroup1',),
                         'gidnumber': [fuzzy_digits],
@@ -454,9 +439,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group2),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group2),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': [user1, user2],
                         'memberof_group': [group1],
                         'gidnumber': [fuzzy_digits],
@@ -481,9 +465,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group3),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group3),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': [user3],
                         'memberof_group': [group1],
                         'gidnumber': [fuzzy_digits],
@@ -508,9 +491,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group3),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group3),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': [user3],
                         'memberof_group': [group1],
                         'member_group': [group4],
@@ -536,9 +518,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group4),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group4),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': [user1],
                         'memberof_group': [group3],
                         'memberofindirect_group': [group1],
@@ -564,9 +545,8 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',group4),('cn','groups'),('cn','accounts'),
-                               api.env.basedn),
+                        'dn': DN(('cn',group4),('cn','groups'),('cn','accounts'),
+                                 api.env.basedn),
                         'member_user': [user1, user4],
                         'memberof_group': [group3],
                         'memberofindirect_group': [group1],
@@ -591,9 +571,8 @@ class test_nesting(Declarative):
                     memberindirect_group = [group4],
                     member_group = [group2, group3],
                     memberindirect_user = [user1, user2, user3, user4],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup1'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup1'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -611,9 +590,8 @@ class test_nesting(Declarative):
                     gidnumber= [fuzzy_digits],
                     memberof_group = [group1],
                     member_user = [user1, user2],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup2'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup2'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -633,9 +611,8 @@ class test_nesting(Declarative):
                     member_user = [user3],
                     member_group = [group4],
                     memberindirect_user = [user1, user4],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup3'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup3'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -654,9 +631,8 @@ class test_nesting(Declarative):
                     memberof_group = [group3],
                     member_user = [user1, user4],
                     memberofindirect_group = [group1],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testgroup4'),('cn','groups'),
-                           ('cn','accounts'),api.env.basedn),
+                    dn=DN(('cn','testgroup4'),('cn','groups'),
+                          ('cn','accounts'),api.env.basedn),
                 ),
             ),
         ),
@@ -676,7 +652,7 @@ class test_nesting(Declarative):
                 value=fqdn1,
                 summary=u'Added host "%s"' % fqdn1,
                 result=dict(
-                    dn=lambda x: DN(x) == host_dn1,
+                    dn=host_dn1,
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
@@ -700,14 +676,13 @@ class test_nesting(Declarative):
                 value=hostgroup1,
                 summary=u'Added hostgroup "testhostgroup1"',
                 result=dict(
-                    dn=lambda x: DN(x) == hgdn1,
+                    dn=hgdn1,
                     cn=[hostgroup1],
                     objectclass=objectclasses.hostgroup,
                     description=[u'Test hostgroup 1'],
                     ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',hostgroup1),('cn','ng'),('cn','alt'),
-                            api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',hostgroup1),('cn','ng'),('cn','alt'),
+                                        api.env.basedn)],
                 ),
             ),
         ),
@@ -722,14 +697,13 @@ class test_nesting(Declarative):
                 value=hostgroup2,
                 summary=u'Added hostgroup "testhostgroup2"',
                 result=dict(
-                    dn=lambda x: DN(x) == hgdn2,
+                    dn=hgdn2,
                     cn=[hostgroup2],
                     objectclass=objectclasses.hostgroup,
                     description=[u'Test hostgroup 2'],
                     ipauniqueid=[fuzzy_uuid],
-                    mepmanagedentry=lambda x: [DN(i) for i in x] == \
-                        [DN(('cn',hostgroup2),('cn','ng'),('cn','alt'),
-                            api.env.basedn)],
+                    mepmanagedentry=[DN(('cn',hostgroup2),('cn','ng'),('cn','alt'),
+                                        api.env.basedn)],
                 ),
             ),
         ),
@@ -749,7 +723,7 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                    'dn': lambda x: DN(x) == hgdn2,
+                    'dn': hgdn2,
                     'cn': [hostgroup2],
                     'description': [u'Test hostgroup 2'],
                     'member_host': [fqdn1],
@@ -772,10 +746,11 @@ class test_nesting(Declarative):
                     ),
                 ),
                 result={
-                    'dn': lambda x: DN(x) == hgdn1,
+                    'dn': hgdn1,
                     'cn': [hostgroup1],
                     'description': [u'Test hostgroup 1'],
                     'member_hostgroup': [hostgroup2],
+                    'memberindirect_host': [fqdn1],
                 },
             ),
         ),
@@ -788,7 +763,7 @@ class test_nesting(Declarative):
                 value=hostgroup1,
                 summary=None,
                 result={
-                    'dn': lambda x: DN(x) == hgdn1,
+                    'dn': hgdn1,
                     'memberindirect_host': [u'testhost1.%s' % api.env.domain],
                     'member_hostgroup': [hostgroup2],
                     'cn': [hostgroup1],
@@ -805,7 +780,7 @@ class test_nesting(Declarative):
                 value=fqdn1,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == host_dn1,
+                    dn=host_dn1,
                     fqdn=[fqdn1],
                     description=[u'Test host 1'],
                     l=[u'Undisclosed location 1'],
