@@ -23,17 +23,40 @@ Test the `ipalib/plugins/sudocmdgroup.py` module.
 from ipalib import api, errors
 from tests.test_xmlrpc import objectclasses
 from xmlrpc_test import Declarative, fuzzy_digits, fuzzy_uuid
-from ipalib.dn import *
+from ipapython.dn import DN
 
 sudocmdgroup1 = u'testsudocmdgroup1'
 sudocmdgroup2 = u'testsudocmdgroup2'
 sudocmd1 = u'/usr/bin/sudotestcmd1'
+sudocmd_plus = u'/bin/ls -l /lost+found/*'
+
+def create_command(sudocmd):
+    return dict(
+        desc='Create %r' % sudocmd,
+        command=(
+            'sudocmd_add', [], dict(sudocmd=sudocmd,
+                description=u'Test sudo command')
+        ),
+        expected=dict(
+            value=sudocmd,
+            summary=u'Added Sudo Command "%s"' % sudocmd,
+            result=dict(
+                objectclass=objectclasses.sudocmd,
+                sudocmd=[sudocmd],
+                ipauniqueid=[fuzzy_uuid],
+                description=[u'Test sudo command'],
+                dn=DN(('sudocmd',sudocmd),('cn','sudocmds'),('cn','sudo'),
+                      api.env.basedn),
+            ),
+        ),
+    )
 
 class test_sudocmdgroup(Declarative):
     cleanup_commands = [
         ('sudocmdgroup_del', [sudocmdgroup1], {}),
         ('sudocmdgroup_del', [sudocmdgroup2], {}),
         ('sudocmd_del', [sudocmd1], {}),
+        ('sudocmd_del', [sudocmd_plus], {}),
     ]
 
     tests = [
@@ -53,9 +76,8 @@ class test_sudocmdgroup(Declarative):
                     sudocmd=[u'/usr/bin/sudotestcmd1'],
                     ipauniqueid=[fuzzy_uuid],
                     description=[u'Test sudo command 1'],
-                    dn=lambda x: DN(x) == \
-                        DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
-                           api.env.basedn),
+                    dn=DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
+                          api.env.basedn),
                 ),
             ),
         ),
@@ -70,9 +92,8 @@ class test_sudocmdgroup(Declarative):
                 result=dict(
                     sudocmd=[sudocmd1],
                     description=[u'Test sudo command 1'],
-                    dn=lambda x: DN(x) == \
-                        DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
-                           api.env.basedn),
+                    dn=DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
+                          api.env.basedn),
                 ),
             ),
         ),
@@ -83,7 +104,8 @@ class test_sudocmdgroup(Declarative):
         dict(
             desc='Try to retrieve non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_show', [sudocmdgroup1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
@@ -91,14 +113,16 @@ class test_sudocmdgroup(Declarative):
             desc='Try to update non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_mod', [sudocmdgroup1],
                 dict(description=u'Foo')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
         dict(
             desc='Try to delete non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_del', [sudocmdgroup1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
@@ -116,9 +140,8 @@ class test_sudocmdgroup(Declarative):
                     description=[u'Test desc 1'],
                     objectclass=objectclasses.sudocmdgroup,
                     ipauniqueid=[fuzzy_uuid],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
             ),
         ),
@@ -130,7 +153,8 @@ class test_sudocmdgroup(Declarative):
                 'sudocmdgroup_add', [sudocmdgroup1],
                 dict(description=u'Test desc 1')
             ),
-            expected=errors.DuplicateEntry(),
+            expected=errors.DuplicateEntry(message=u'sudo command group ' +
+                u'with name "%s" already exists' % sudocmdgroup1),
         ),
 
 
@@ -143,9 +167,8 @@ class test_sudocmdgroup(Declarative):
                 result=dict(
                     cn=[sudocmdgroup1],
                     description=[u'Test desc 1'],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
             ),
         ),
@@ -176,9 +199,8 @@ class test_sudocmdgroup(Declarative):
                 result=dict(
                     cn=[sudocmdgroup1],
                     description=[u'New desc 1'],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup1'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
                 summary=None,
             ),
@@ -193,9 +215,8 @@ class test_sudocmdgroup(Declarative):
                 truncated=False,
                 result=[
                     dict(
-                        dn=lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        dn=DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                              ('cn','sudo'),api.env.basedn),
                         cn=[sudocmdgroup1],
                         description=[u'New desc 1'],
                     ),
@@ -211,7 +232,8 @@ class test_sudocmdgroup(Declarative):
         dict(
             desc='Try to retrieve non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_show', [sudocmdgroup2], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
@@ -219,14 +241,16 @@ class test_sudocmdgroup(Declarative):
             desc='Try to update non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_mod', [sudocmdgroup2],
                 dict(description=u'Foo')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
         dict(
             desc='Try to delete non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_del', [sudocmdgroup2], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
@@ -244,9 +268,8 @@ class test_sudocmdgroup(Declarative):
                     description=[u'Test desc 2'],
                     objectclass=objectclasses.sudocmdgroup,
                     ipauniqueid=[fuzzy_uuid],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
             ),
         ),
@@ -258,7 +281,9 @@ class test_sudocmdgroup(Declarative):
                 'sudocmdgroup_add', [sudocmdgroup2],
                 dict(description=u'Test desc 2')
             ),
-            expected=errors.DuplicateEntry(),
+            expected=errors.DuplicateEntry(
+                message=u'sudo command group with name "%s" already exists' %
+                    sudocmdgroup2),
         ),
 
 
@@ -271,9 +296,8 @@ class test_sudocmdgroup(Declarative):
                 result=dict(
                     cn=[sudocmdgroup2],
                     description=[u'Test desc 2'],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
             ),
         ),
@@ -304,9 +328,8 @@ class test_sudocmdgroup(Declarative):
                 result=dict(
                     cn=[sudocmdgroup2],
                     description=[u'New desc 2'],
-                    dn=lambda x: DN(x) == \
-                        DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    dn=DN(('cn','testsudocmdgroup2'),('cn','sudocmdgroups'),
+                          ('cn','sudo'),api.env.basedn),
                 ),
                 summary=None,
             ),
@@ -321,9 +344,8 @@ class test_sudocmdgroup(Declarative):
                 truncated=False,
                 result=[
                     dict(
-                        dn=lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup2),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        dn=DN(('cn',sudocmdgroup2),('cn','sudocmdgroups'),
+                              ('cn','sudo'),api.env.basedn),
                         cn=[sudocmdgroup2],
                         description=[u'New desc 2'],
                     ),
@@ -342,16 +364,14 @@ class test_sudocmdgroup(Declarative):
                 truncated=False,
                 result=[
                     dict(
-                        dn=lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        dn=DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                              ('cn','sudo'),api.env.basedn),
                         cn=[sudocmdgroup1],
                         description=[u'New desc 1'],
                     ),
                     dict(
-                        dn=lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup2),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        dn=DN(('cn',sudocmdgroup2),('cn','sudocmdgroups'),
+                              ('cn','sudo'),api.env.basedn),
                         cn=[sudocmdgroup2],
                         description=[u'New desc 2'],
                     ),
@@ -377,9 +397,8 @@ class test_sudocmdgroup(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                                 ('cn','sudo'),api.env.basedn),
                         'member_sudocmd': (sudocmd1,),
                         'cn': [sudocmdgroup1],
                         'description': [u'New desc 1'],
@@ -394,9 +413,8 @@ class test_sudocmdgroup(Declarative):
                 value=sudocmd1,
                 summary=None,
                 result=dict(
-                    dn=lambda x: DN(x) == \
-                        DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
-                           api.env.basedn),
+                    dn=DN(('sudocmd',sudocmd1),('cn','sudocmds'),('cn','sudo'),
+                          api.env.basedn),
                     sudocmd=[sudocmd1],
                     description=[u'Test sudo command 1'],
                     memberof_sudocmdgroup = [u'testsudocmdgroup1'],
@@ -418,9 +436,8 @@ class test_sudocmdgroup(Declarative):
                     ),
                 ),
                 result={
-                        'dn': lambda x: DN(x) == \
-                            DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                               ('cn','sudo'),api.env.basedn),
+                        'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                                 ('cn','sudo'),api.env.basedn),
                         'member_sudocmd': (u'/usr/bin/sudotestcmd1',),
                         'cn': [sudocmdgroup1],
                         'description': [u'New desc 1'],
@@ -441,9 +458,8 @@ class test_sudocmdgroup(Declarative):
                     ),
                 ),
                 result={
-                    'dn': lambda x: DN(x) == \
-                        DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                             ('cn','sudo'),api.env.basedn),
                     'cn': [sudocmdgroup1],
                     'description': [u'New desc 1'],
                 },
@@ -464,15 +480,60 @@ class test_sudocmdgroup(Declarative):
                     ),
                 ),
                 result={
-                    'dn': lambda x: DN(x) == \
-                        DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
-                           ('cn','sudo'),api.env.basedn),
+                    'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                             ('cn','sudo'),api.env.basedn),
                     'cn': [sudocmdgroup1],
                     'description': [u'New desc 1'],
                 },
             ),
         ),
 
+        ################
+        # test a command that needs DN escaping:
+        create_command(sudocmd_plus),
+
+        dict(
+            desc='Add %r to %r' % (sudocmd_plus, sudocmdgroup1),
+            command=('sudocmdgroup_add_member', [sudocmdgroup1],
+                dict(sudocmd=sudocmd_plus)
+            ),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    member=dict(
+                        sudocmd=tuple(),
+                    ),
+                ),
+                result={
+                        'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                                 ('cn','sudo'),api.env.basedn),
+                        'member_sudocmd': (sudocmd_plus,),
+                        'cn': [sudocmdgroup1],
+                        'description': [u'New desc 1'],
+                },
+            ),
+        ),
+
+        dict(
+            desc='Remove %r from %r' %  (sudocmd_plus, sudocmdgroup1),
+            command=('sudocmdgroup_remove_member', [sudocmdgroup1],
+                dict(sudocmd=sudocmd_plus)
+            ),
+            expected=dict(
+                completed=1,
+                failed=dict(
+                    member=dict(
+                        sudocmd=tuple(),
+                    ),
+                ),
+                result={
+                        'dn': DN(('cn',sudocmdgroup1),('cn','sudocmdgroups'),
+                                 ('cn','sudo'),api.env.basedn),
+                        'cn': [sudocmdgroup1],
+                        'description': [u'New desc 1'],
+                },
+            ),
+        ),
 
         ################
         # delete sudocmdgroup1:
@@ -490,14 +551,16 @@ class test_sudocmdgroup(Declarative):
         dict(
             desc='Try to delete non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_del', [sudocmdgroup1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
         dict(
             desc='Try to retrieve non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_show', [sudocmdgroup1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
@@ -505,7 +568,8 @@ class test_sudocmdgroup(Declarative):
             desc='Try to update non-existent %r' % sudocmdgroup1,
             command=('sudocmdgroup_mod', [sudocmdgroup1],
             dict(description=u'Foo')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup1),
         ),
 
 
@@ -525,14 +589,16 @@ class test_sudocmdgroup(Declarative):
         dict(
             desc='Try to delete non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_del', [sudocmdgroup2], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
         dict(
             desc='Try to retrieve non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_show', [sudocmdgroup2], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
@@ -540,7 +606,8 @@ class test_sudocmdgroup(Declarative):
             desc='Try to update non-existent %r' % sudocmdgroup2,
             command=('sudocmdgroup_mod', [sudocmdgroup2],
             dict(description=u'Foo')),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command group not found' % sudocmdgroup2),
         ),
 
 
@@ -560,7 +627,8 @@ class test_sudocmdgroup(Declarative):
         dict(
             desc='Verify that %r is really gone' % sudocmd1,
             command=('sudocmd_show', [sudocmd1], {}),
-            expected=errors.NotFound(reason='no such entry'),
+            expected=errors.NotFound(
+                reason=u'%s: sudo command not found' % sudocmd1),
         ),
 
     ]
